
from __future__ import absolute_import

from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from leonardo.views import *
from leonardo import messages

from .forms import (get_widget_create_form, get_widget_update_form,
                    WidgetDeleteForm, WidgetSelectForm, WidgetUpdateForm)
from .tables import WidgetDimensionTable
from .utils import get_widget_from_id


class WidgetViewMixin(object):

    def handle_dimensions(self, obj):
        """save dimensions
        """
        from .tables import WidgetDimensionFormset
        from ..models import WidgetDimension
        formset = WidgetDimensionFormset(
            self.request.POST, prefix='dimensions')
        for form in formset.forms:
            if form.is_valid():
                if 'id' in form.cleaned_data:
                    form.save()
            else:
                # little ugly
                data = form.cleaned_data
                data['widget_type'] = \
                    ContentType.objects.get_for_model(obj)
                data['widget_id'] = obj.id
                data.pop('DELETE', None)
                wd = WidgetDimension(**data)
                wd.save()
        # optionaly delete dimensions
        if formset.is_valid():
            formset.save(commit=False)
            # delete objects
            for obj in formset.deleted_objects:
                obj.delete()
        return True


class WidgetUpdateView(UpdateView, WidgetViewMixin):

    template_name = 'leonardo/common/modal.html'

    form_class = WidgetUpdateForm

    def get_context_data(self, **kwargs):
        context = super(WidgetUpdateView, self).get_context_data(**kwargs)
        return context

    def get_form(self, form_class):
        """Returns an instance of the form to be used in this view."""

        kwargs = self.get_form_kwargs()
        kwargs.update({
            'request': self.request,
        })
        return get_widget_update_form(**self.kwargs)(**kwargs)

    def form_valid(self, form):
        response = super(WidgetUpdateView, self).form_valid(form)
        obj = self.object
        self.handle_dimensions(obj)
        return response


class WidgetCreateView(CreateView, WidgetViewMixin):

    template_name = 'leonardo/common/modal.html'

    def get_label(self):
        return ugettext("Create new Widget")

    def get_form(self, form_class):
        kwargs = self.get_form_kwargs()
        return get_widget_create_form(**self.kwargs)(**kwargs)

    def get_context_data(self, **kwargs):
        context = super(WidgetCreateView, self).get_context_data(**kwargs)
        context['table'] = WidgetDimensionTable(self.request, data=[])
        # add extra context for template
        context['url'] = reverse("widget_create_full", kwargs=self.kwargs)
        return context

    def form_valid(self, form):
        try:
            obj = form.save(commit=False)
            obj.save()
            obj.parent.save()
            success_url = self.get_success_url()
            response = HttpResponseRedirect(success_url)
            response['X-Horizon-Location'] = success_url
        except Exception as e:
            raise e

        return response

    def get_initial(self):
        return self.kwargs


class WidgetPreCreateView(CreateView):

    form_class = WidgetSelectForm

    template_name = 'leonardo/common/modal.html'

    def get_label(self):
        return ugettext("Create new Widget")

    def get_context_data(self, **kwargs):
        context = super(WidgetPreCreateView, self).get_context_data(**kwargs)
        context['modal_size'] = 'sm'
        context['form_submit'] = _('Continue')
        return context

    def get_form(self, form_class):
        """Returns an instance of the form to be used in this view."""
        kwargs = self.kwargs
        kwargs.update(self.get_form_kwargs())
        kwargs.update({
            'request': self.request,
            'next_view': WidgetCreateView
        })
        return form_class(**kwargs)


class WidgetInfoView(UpdateView, WidgetViewMixin):

    template_name = 'leonardo/common/modal.html'

    form_class = WidgetUpdateForm

    def get(self, request, cls_name, id):

        widget = self.object

        widget_info = """
            <ul>
                <li><span><b>widget:</b>&nbsp;{name}&nbsp({id})</span></li>
                <li><span><b>parent:</b>&nbsp;{parent}&nbsp({parent_id})</span></li>
                <li><span><b>region:</b>&nbsp;{region}</span></li>
                <li><span><b>ordering:</b>&nbsp;{ordering}</span></li>
            </ul>""".format(**{
            'name': widget.__class__.__name__,
            'id': widget.id,
            'region': widget.region,
            'parent': widget.parent,
            'parent_id': widget.parent.pk,
            'ordering': widget.ordering,
        })

        messages.info(request, mark_safe(widget_info))

        return HttpResponse(mark_safe(widget_info))


class WidgetDeleteView(ModalFormView, ContextMixin, ModelFormMixin):

    form_class = WidgetDeleteForm

    template_name = 'leonardo/common/modal.html'

    def get_label(self):
        return ugettext("Delete")  # .format(self.object.label))

    def get_context_data(self, **kwargs):
        context = super(WidgetDeleteView, self).get_context_data(**kwargs)

        # add extra context for template
        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = self.get_header()
        context['title'] = self.get_header()
        context['form_submit'] = self.get_label()
        context['heading'] = self.get_header()
        context['help_text'] = self.get_help_text()
        return context

    def form_valid(self, form):
        obj = self.object
        try:
            parent = obj.parent
            obj.delete()
            # invalide page cache
            parent.invalidate_cache()
            success_url = parent.get_absolute_url()
            response = HttpResponseRedirect(success_url)
            response['X-Horizon-Location'] = success_url
        except Exception as e:
            raise e

        return response

    def get_initial(self):
        return self.kwargs


class WidgetSortView(ModalFormView):

    '''Simple handle jquery sortable'''

    template_name = 'leonardo/common/modal.html'

    form_class = WidgetUpdateForm

    def get(self, *args, **kwargs):
        return self.post(*args, **kwargs)

    def post(self, *args, **kwargs):

        widgets = self.request.POST.getlist('widgets[]', [])

        widget_list = []

        try:
            for widget_id in widgets:
                widget = get_widget_from_id(widget_id)
                if widget:
                    widget_list.append(widget)
        except:
            messages.error(
                self.request, _('Error occured while sorting widgets.'))

        i = 0

        for widget in widget_list:
            widget.ordering = i
            widget.save()
            i += 1

        messages.success(self.request, _('Widget sorting success.'))

        return HttpResponse('ok')


class WidgetReorderView(ModalFormView, ModelFormMixin):

    '''Handle reorder 0 = first, 1 = last'''

    template_name = 'leonardo/common/modal.html'

    form_class = WidgetUpdateForm

    def get(self, *args, **kwargs):
        return self.post(*args, **kwargs)

    def post(self, *args, **kwargs):

        widget = self.object

        ordering = self.kwargs.get('ordering')

        if int(ordering) == 0:

            widget.ordering = 0
            widget.save()

            widgets = getattr(widget.parent.content, widget.region)
            widgets = [w for w in widgets if w.id != widget.id]

            for i, _widget in enumerate(widgets):
                _widget.ordering = i + 1
                _widget.save()

        elif int(ordering) == -1:
            next_ordering = widget.ordering - 1
            widgets = getattr(widget.parent.content, widget.region)
            for w in widgets:
                if w.ordering == next_ordering:
                    w.ordering = widget.ordering
                    w.save()
                    widget.ordering = next_ordering
                    widget.save()
        elif int(ordering) == 1:

            next_ordering = widget.ordering + 1
            widgets = getattr(widget.parent.content, widget.region)
            for w in widgets:
                if w.ordering == next_ordering:
                    w.ordering = widget.ordering
                    w.save()
                    widget.ordering = next_ordering
                    widget.save()

        else:
            widget.ordering = widget.next_ordering
            widget.save()
            widgets = getattr(widget.parent.content, widget.region)
            widgets = [w for w in widgets if w.id != widget.id]
            widgets.sort(key=lambda w: w.ordering)

            for i, _widget in enumerate(widgets):
                _widget.ordering = i
                _widget.save()

        messages.success(self.request, _('Widget was successfully moved.'))

        # TODO try HTTP_REFERER
        success_url = widget.parent.get_absolute_url()
        response = HttpResponseRedirect(success_url)
        response['X-Horizon-Location'] = success_url
        return response




from leonardo.views import ModalFormView
from leonardo.forms import SelfHandlingForm


class FakeForm(SelfHandlingForm):

    def handle(self, request, data):
        pass


class ModalIframeView(ModalFormView):

    form_class = FakeForm

    template_name = 'devel/modal_iframe.html'

    def get_context_data(self, *args, **kwargs):

        context = super(ModalFormView, self).get_context_data(*args, **kwargs)

        context['url'] = self.kwargs['url'] + '?_popup=1'
        context['modal_size'] = 'lg'
        return context




import floppyforms as forms
from django.forms.models import modelformset_factory
from django.utils.translation import ugettext_lazy as _
from horizon import tables
from horizon.tables.formset import FormsetDataTable, FormsetRow
from leonardo.module.web.models import WidgetDimension


class Slider(forms.RangeInput):
    min = 1
    max = 12
    step = 1
    template_name = 'floppyforms/slider.html'


class OffsetSlider(Slider):
    min = 0


class WidgetDimensionForm(forms.ModelForm):

    width = forms.CharField(widget=Slider(), initial=12)
    height = forms.CharField(widget=OffsetSlider(), initial=0)
    offset = forms.CharField(widget=OffsetSlider(), initial=0)

    class Meta:
        model = WidgetDimension
        exclude = tuple()

WidgetDimensionFormset = modelformset_factory(
    WidgetDimension, form=WidgetDimensionForm, can_delete=True, extra=1)


class CustomFormsetRow(FormsetRow):

    def __init__(self, column, datum, form):
        self.form = form
        super(CustomFormsetRow, self).__init__(column, datum, form)
        # add initial
        if not datum and column.data:
            try:
                previous = column.data[0]
                self.form.fields['widget_type'].initial = previous.widget_type
                self.form.fields['widget_id'].initial = previous.widget_id
                self.form.fields['id'].initial = previous.id + 1
            except Exception:
                pass


class WidgetDimensionTable(FormsetDataTable):

    formset_class = WidgetDimensionFormset

    def get_formset(self):
        """Provide the formset corresponding to this DataTable.

        Use this to validate the formset and to get the submitted data back.
        """
        if self.widget:
            queryset = self.widget.dimensions
        else:
            queryset = WidgetDimension.objects.none()
        if self._formset is None:
            self._formset = self.formset_class(
                self.request.POST or None,
                initial=self._get_formset_data(),
                prefix=self._meta.name,
                queryset=queryset)
        return self._formset

    def __init__(self, *args, **kwargs):
        self._meta.row_class = CustomFormsetRow
        self.widget = kwargs.pop('widget', None)
        super(WidgetDimensionTable, self).__init__(*args, **kwargs)

    widget_id = tables.Column('widget_id', hidden=True)
    widget_type = tables.Column('widget_type', hidden=True)
    size = tables.Column('size', verbose_name=_('Size'))
    width = tables.Column('width', verbose_name=('Width'))
    height = tables.Column('height', verbose_name=_('Height'))
    offset = tables.Column('offset', verbose_name=_('Offset'))

    name = 'dimensions'

    class Meta:
        name = 'dimensions'
        table_name = 'Dimensions'




from __future__ import absolute_import

from leonardo.module.web.widget.application.models import ApplicationWidget
from leonardo.module.web.widget.application.reverse import (app_reverse,
                                                            app_reverse_lazy)

try:
    from feincms.views.decorators import standalone, unpack
    from feincms.content.application.models import permalink
except:
    from feincms.apps import standalone, permalink, unpack

from .leonardo import *

__all__ = (
    'ApplicationWidget',
    'app_reverse', 'app_reverse_lazy', 'permalink',
    'standalone', 'unpack',
)




from __future__ import absolute_import

import functools
from django.conf import settings
from datetime import datetime, timedelta

CACHE_EXPIRATION = getattr(settings, 'CACHE_EXPIRATION', 3600)
LEONARDO_MEMOIZED = getattr(settings, 'LEONARDO_MEMOIZED', True)


class memoized(object):

    """Decorator. Caches a function's return value each time it is called.
    If called later with the same arguments, the cached value is returned
    (not reevaluated), unless more than expiration passed.
    """

    def __init__(self, func):
        self.func = func
        self.cache = {}

    def is_actual(self, id):
        expirated = datetime.now() - timedelta(seconds=CACHE_EXPIRATION)
        if id in self.cache \
                and self.cache[id][0] >= expirated:
            return True
        return False

    def __call__(self, *args):
        if not LEONARDO_MEMOIZED:
            return self.func(*args)
        instance = args[0]
        id = "{}-{}-{}".format(
            instance._meta.app_label,
            instance.__class__.__name__,
            instance.id)
        if self.is_actual(id):
            return self.cache[id][1]
        else:
            content = self.func(*args)
            self.cache[id] = datetime.now(), content
            return content

    def __repr__(self):
        '''Return the function's docstring.'''
        return self.func.__doc__

    def __get__(self, obj, objtype):
        '''Support instance methods.'''
        return functools.partial(self.__call__, obj)


class widget_memoized(object):

    """Designed only for widget render function.

        note: requires Leonardo Web Middleware
    """

    def __init__(self, func):
        self.func = func
        self.cache = {}

    def is_actual(self, id):
        expirated = datetime.now() - timedelta(seconds=CACHE_EXPIRATION)
        if id in self.cache \
                and self.cache[id][0] >= expirated:
            return True
        return False

    def __call__(self, *args):

        if not LEONARDO_MEMOIZED:
            return self.func(*args)

        instance = args[0]

        try:
            request = args[1]['request']

            id = "{}-{}-{}-{}-{}-{}".format(
                instance._meta.app_label,
                instance.__class__.__name__,
                instance.id,
                request.user,
                request.leonardo_page.slug
            )

            if self.is_actual(id) and not getattr(instance, 'saved', False) \
                    and not request.frontend_editing:
                return self.cache[id][1]
            else:
                content = self.func(*args)
                self.cache[id] = datetime.now(), content
                return content
        except Exception:
            # all process is optional if failed nothing to do
            return self.func(*args)

    def __repr__(self):
        '''Return the function's docstring.'''
        return self.func.__doc__

    def __get__(self, obj, objtype):
        '''Support instance methods.'''
        return functools.partial(self.__call__, obj)



# -#- coding: utf-8 -#-

import datetime

import feedparser
from django.db import models
from django.template.context import RequestContext
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo.module.web.models import Widget, ContentProxyWidgetMixin
from leonardo.module.web.widgets.mixins import ListWidgetMixin

TARGET_CHOICES = (
    ('modal', _('Modal window')),
    ('blank', _('Blank window')),
)


class FeedReaderWidget(Widget, ContentProxyWidgetMixin, ListWidgetMixin):
    max_items = models.IntegerField(_('max. items'), default=5)

    class Meta:
        abstract = True
        verbose_name = _("feed reader")
        verbose_name_plural = _('feed readers')

    def render_content(self, options):

        if self.is_obsolete:
            self.update_cache_data()

        context = RequestContext(options.get('request'), {
            'widget': self,
        })

        return render_to_string(self.get_template_name(), context)

    def update_cache_data(self, save=True):

        feed = feedparser.parse(self.link)
        entries = feed['entries'][:self.max_items]

        context = {
            'widget': self,
            'link': feed['feed']['link'],
            'entries': entries,
        }

        self.cache_data = render_to_string(
            'widget/feedreader/_content.html', context)
        self.cache_update = datetime.datetime.now()

        if save:
            self.save()

    def save(self, *args, **kwargs):
        self.update_cache_data(False)
        super(FeedReaderWidget, self).save(*args, **kwargs)



from __future__ import unicode_literals

import sys

from django.conf import settings
from django.template.base import Library
from django.utils import six
from django.utils.encoding import smart_text
from leonardo.module.web.widget import ApplicationWidget
from leonardo.module.web.widget.application.reverse import app_reverse as do_app_reverse

register = Library()


def render(self, context):
    from django.core.urlresolvers import reverse, NoReverseMatch
    args = [arg.resolve(context) for arg in self.args]
    kwargs = {
        smart_text(k, 'ascii'): v.resolve(context)
        for k, v in self.kwargs.items()
    }

    view_name = self.view_name.resolve(context)

    try:
        current_app = context.request.current_app
    except AttributeError:
        # Change the fallback value to None when the deprecation path for
        # Context.current_app completes in Django 2.0.
        current_app = context.current_app

    # Try to look up the URL twice: once given the view name, and again
    # relative to what we guess is the "main" app. If they both fail,
    # re-raise the NoReverseMatch unless we're using the
    # {% url ... as var %} construct in which case return nothing.
    url = ''

    try:
        url = reverse(
            view_name, args=args, kwargs=kwargs, current_app=current_app)
    except NoReverseMatch:
        # try external apps
        for urlconf, config in six.iteritems(
                ApplicationWidget._feincms_content_models[0].ALL_APPS_CONFIG):
            partials = view_name.split(':')[1:]
            try:
                url = do_app_reverse(
                    ':'.join(partials), urlconf, args=args, kwargs=kwargs,
                    current_app=context.current_app)
            except NoReverseMatch:
                pass
            else:
                return url

        exc_info = sys.exc_info()
        if settings.SETTINGS_MODULE:
            project_name = settings.SETTINGS_MODULE.split('.')[0]
            try:
                url = reverse(project_name + '.' + view_name,
                              args=args, kwargs=kwargs,
                              current_app=current_app)
            except NoReverseMatch:
                if self.asvar is None:
                    # Re-raise the original exception, not the one with
                    # the path relative to the project. This makes a
                    # better error message.
                    six.reraise(*exc_info)
        else:
            if self.asvar is None:
                raise

    if self.asvar:
        context[self.asvar] = url
        return ''
    else:
        return url




from django.contrib import admin

from .clipboardadmin import ClipboardAdmin
from .fileadmin import FileAdmin
from .folderadmin import FolderAdmin
from .imageadmin import ImageAdmin
from .permissionadmin import PermissionAdmin
from ..models import File, FolderPermission, Folder, Clipboard, Image


admin.site.register(File, FileAdmin)
admin.site.register(Folder, FolderAdmin)
admin.site.register(Clipboard, ClipboardAdmin)
admin.site.register(Image, ImageAdmin)
admin.site.register(FolderPermission, PermissionAdmin)

from ..models import Document, Video, Vector
# common stuff
from .mediaadmin import DocumentAdmin, VectorAdmin, VideoAdmin

admin.site.register(Video, VideoAdmin)
admin.site.register(Document, DocumentAdmin)
admin.site.register(Vector, VectorAdmin)




from django.apps import AppConfig

from .widget import *


default_app_config = 'leonardo.module.nav.NavConfig'


class Default(object):

    optgroup = ('Navigation')

    apps = ['leonardo.module.nav']

    @property
    def widgets(self):
        return [
            TreeNavigationWidget,
            ContentNavigationWidget,
            ContextNavigationWidget,
            LinearNavigationWidget,
            BreadcrumbsWidget,
            SiteMapWidget,
            SiteSearchWidget,
            LanguageSelectorWidget
        ]


class NavConfig(AppConfig, Default):
    name = 'leonardo.module.nav'
    verbose_name = "Navigation Module"

default = Default()




from django.http import HttpResponseRedirect
from django.utils.cache import add_never_cache_headers


def frontendediting_request_processor(page, request):
    """
    Sets the frontend editing state in the cookie depending on the
    ``frontend_editing`` GET parameter and the user's permissions.
    """
    if 'frontend_editing' not in request.GET:
        return

    response = HttpResponseRedirect(request.path)

    if request.user.has_module_perms('page'):
        try:
            enable_fe = int(request.GET['frontend_editing']) > 0
        except ValueError:
            enable_fe = False

        if enable_fe:
            response.set_cookie(str('frontend_editing'), enable_fe)
        else:
            response.delete_cookie(str('frontend_editing'))

    # Redirect to cleanup URLs
    return response


def frontendediting_response_processor(page, request, response):
    # Add never cache headers in case frontend editing is active
    if (hasattr(request, 'COOKIES')
            and request.COOKIES.get('frontend_editing', False)):

        if hasattr(response, 'add_post_render_callback'):
            response.add_post_render_callback(add_never_cache_headers)
        else:
            add_never_cache_headers(response)




from django.contrib import admin
from .. import settings
from ..fields import folder


class PermissionAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {'fields': (('type', 'folder',))}),
        (None, {'fields': (('user', 'group', 'everybody'),)}),
        (None, {'fields': (
            ('can_edit', 'can_read', 'can_add_children')
        )}
        ),
    )
    raw_id_fields = ('user', 'group',)
    list_filter = ['user']
    list_display = ['__str__', 'folder', 'user']

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        db = kwargs.get('using')
        if db_field.name == 'folder':
            kwargs['widget'] = folder.AdminFolderWidget(
                db_field.rel, self.admin_site, using=db)
        return super(PermissionAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def get_model_perms(self, request):
        # don't display the permissions admin if permissions are disabled.
        # This method is easier for testing than not registering the admin at all
        # at import time
        enable_permissions = settings.MEDIA_ENABLE_PERMISSIONS and request.user.has_perm(
            'media.add_folderpermission')
        return {
            'add': enable_permissions,
            'change': enable_permissions,
            'delete': enable_permissions,
        }




from django.db.models.fields import BLANK_CHOICE_DASH
from crispy_forms.bootstrap import Tab, TabHolder
from crispy_forms.layout import Layout
from django.http import HttpResponseRedirect
from leonardo import forms
from django.utils.translation import ugettext_lazy as _
from leonardo.forms import SelfHandlingForm
from leonardo.module.web.page.fields import (
                                             PageColorSchemeSelectField,
                                             PageThemeSelectField)
from leonardo.module.web.models import Page
from leonardo.module.web.const import PAGE_LAYOUT_CHOICES


class PageMassChangeForm(SelfHandlingForm):

    """Page Mass Update

    Form for mass update of page theme, color scheme and layout

    """

    page_id = forms.IntegerField(
        label=_('Page ID'), widget=forms.widgets.HiddenInput)

    color_scheme = PageColorSchemeSelectField(
        label=_('Color Scheme'),
        required=False
    )

    theme = PageThemeSelectField(
        label=_('Theme'),
        required=False
    )

    layout = forms.ChoiceField(
        label=_('Layout'), choices=BLANK_CHOICE_DASH + list(PAGE_LAYOUT_CHOICES),
        required=False)

    depth = forms.IntegerField(label=_('Depth'), initial=1)
    from_root = forms.BooleanField(label=_('From Root ?'), initial=True)

    def __init__(self, *args, **kwargs):
        super(PageMassChangeForm, self).__init__(*args, **kwargs)

        self.helper.layout = Layout(
            TabHolder(
                Tab(_('Options'),
                    'depth',
                    'page_id',
                    'from_root',
                    ),
                Tab(_('Styles'),
                    'layout',
                    'theme',
                    'color_scheme',
                    ),
            ),
        )

    def handle(self, request, data):

        root_page = Page.objects.get(pk=data['page_id'])

        if data['from_root']:
            root_page = root_page.get_root()

        color_scheme = data.get('color_scheme', None)
        theme = data.get('theme', None)
        layout = data.get('layout', None)

        if color_scheme:
            root_page.color_scheme = data['color_scheme']
        if layout:
            root_page.layout = data['layout']
        if theme:
            root_page.theme = data['theme']

        for page in root_page.get_descendants():

            if page.level <= data['depth']:
                if color_scheme:
                    page.color_scheme = data['color_scheme']
                if layout:
                    page.layout = data['layout']
                if theme:
                    page.theme = data['theme']

                page.save()

        root_page.save()

        return HttpResponseRedirect(root_page.get_absolute_url())




"""
General-purpose decorators for use with Leonardo.
"""
from __future__ import absolute_import, unicode_literals

import functools
import sys

from django.utils.decorators import available_attrs  # noqa
from django.utils.translation import ugettext_lazy as _

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


def require_auth(view_func):
    """Performs user authentication check.

    Similar to Django's `login_required` decorator, except that this throws
    :exc:`~leonardo.exceptions.NotAuthenticated` exception if the user is not
    signed-in.
    """
    from leonardo.exceptions import NotAuthenticated  # noqa

    @functools.wraps(view_func, assigned=available_attrs(view_func))
    def dec(request, *args, **kwargs):
        if request.user.is_authenticated():
            return view_func(request, *args, **kwargs)
        raise NotAuthenticated(_("Please log in to continue."))
    return dec


def _decorate_urlconf(urlpatterns, decorator, *args, **kwargs):
    '''Decorate all urlpatterns by specified decorator'''

    if isinstance(urlpatterns, (list, tuple)):

        for pattern in urlpatterns:
            if getattr(pattern, 'callback', None):
                pattern._callback = decorator(
                    pattern.callback, *args, **kwargs)
            if getattr(pattern, 'url_patterns', []):
                _decorate_urlconf(
                    pattern.url_patterns, decorator, *args, **kwargs)
    else:
        if getattr(urlpatterns, 'callback', None):
            urlpatterns._callback = decorator(
                urlpatterns.callback, *args, **kwargs)


def catch_result(task_func):
    """Catch printed result from Celery Task and return it in task response
    """

    @functools.wraps(task_func, assigned=available_attrs(task_func))
    def dec(*args, **kwargs):
        # inicialize
        orig_stdout = sys.stdout
        sys.stdout = content = StringIO()
        task_response = task_func(*args, **kwargs)
        # catch
        sys.stdout = orig_stdout
        content.seek(0)
        # propagate to the response
        task_response['stdout'] = content.read()
        return task_response
    return dec




import copy

import floppyforms
from crispy_forms.bootstrap import Tab, TabHolder
from crispy_forms.layout import HTML, Field, Fieldset, Layout
from django import forms
from django.apps import apps
from django.forms.models import modelform_factory
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _
from django_select2.forms import Select2Widget
from feincms.admin.item_editor import ItemEditorForm
from horizon.utils.memoized import memoized
from horizon_contrib.common import get_class
from leonardo.forms import SelfHandlingForm, SelfHandlingModelForm
from leonardo.utils.widgets import get_grouped_widgets


class IconPreviewSelect(floppyforms.widgets.Select):
    template_name = 'floppyforms/select_preview.html'


WIDGETS = {
    'template_name': forms.RadioSelect(choices=[]),
    'parent': forms.widgets.HiddenInput,
    'ordering': forms.widgets.HiddenInput,
    'icon': IconPreviewSelect(attrs={'style': "font-family: 'FontAwesome', Helvetica;"}),
}


class WidgetUpdateForm(ItemEditorForm, SelfHandlingModelForm):

    '''Widget Create/Update Form'''

    id = forms.CharField(
        widget=forms.widgets.HiddenInput,
        required=False
    )

    def __init__(self, *args, **kwargs):
        request = kwargs.pop('request', None)
        super(WidgetUpdateForm, self).__init__(*args, **kwargs)

        if request:
            queryset = self.fields['content_theme'].queryset

            self.fields['content_theme'].queryset = \
                queryset.filter(widget_class=self._meta.model.__name__)
        else:
            # set defaults
            self.init_themes()
            del self.fields['id']

        # get all fields for widget
        main_fields = self._meta.model.fields()
        main_fields.update({'label': 'label'})
        self.helper.layout = Layout(
            TabHolder(
                Tab(self._meta.model._meta.verbose_name.capitalize(),
                    *main_fields,
                    css_id='field-{}'.format(slugify(self._meta.model))
                    ),
                Tab(_('Theme'),
                    'base_theme', 'content_theme', 'color_scheme',
                    Fieldset(_('Positions'), 'layout', 'align',
                             'vertical_align'),
                    'id', 'region', 'ordering',
                    'parent',
                    css_id='theme-widget-settings'
                    ),
                Tab(_('Effects'),
                    'enter_effect_style', 'enter_effect_duration',
                    'enter_effect_delay', 'enter_effect_offset',
                    'enter_effect_iteration',
                    css_id='theme-widget-effects'
                    ),
            )
        )

        self.fields['label'].widget = \
            forms.TextInput(
                attrs={'placeholder': self._meta.model._meta.verbose_name})

        if request:
            from .tables import WidgetDimensionTable
            _request = copy.copy(request)
            _request.POST = {}
            _request.method = 'GET'
            initial = kwargs.get('initial', None)
            if initial and initial.get('id', None):
                widget = self._meta.model.objects.get(
                    id=initial['id'])
                data = widget.dimensions
            else:
                data = []
                widget = None
            dimensions = Tab(_('Dimensions'),
                             HTML(
                                 WidgetDimensionTable(_request, widget=widget, data=data).render()),
                             )
            self.helper.layout[0].append(dimensions)

        # hide label
        if 'text' in self.fields:
            self.fields['text'].label = ''

    def init_themes(self):
        queryset = self.fields['content_theme'].queryset

        self.fields['content_theme'].queryset = \
            queryset.filter(widget_class=self._meta.model.__name__)

        try:
            base_theme = self.fields['base_theme'].queryset.get(
                name__icontains='default')
        except:
            self.fields['base_theme'].initial = \
                self.fields['base_theme'].queryset.first()
        else:
            self.fields['base_theme'].initial = base_theme

        try:
            content_theme = self.fields['content_theme'].queryset.get(
                name__icontains='default')
        except:
            self.fields['content_theme'].initial = \
                self.fields['content_theme'].queryset.first()
        else:
            self.fields['content_theme'].initial = content_theme


class WidgetCreateForm(WidgetUpdateForm):

    ''''''

    pass


class WidgetDeleteForm(SelfHandlingForm):

    def handle(self, request, data):
        pass


class WidgetSelectForm(SelfHandlingForm):

    cls_name = forms.ChoiceField(
        label="Widget Type",
        choices=[],
        required=True,
        widget=Select2Widget()
    )

    first = forms.BooleanField(
        label=('First'),
        help_text=_('If is checked, widget will be'
                    ' placed as first widget in this region'),
        initial=False,
        required=False
    )

    page_id = forms.IntegerField(
        widget=forms.widgets.HiddenInput,
    )

    ordering = forms.IntegerField(
        widget=forms.widgets.HiddenInput,
        initial=0
    )
    region = forms.CharField(
        widget=forms.widgets.HiddenInput,
    )
    parent = forms.IntegerField(
        widget=forms.widgets.HiddenInput,
    )

    def __init__(self, *args, **kwargs):
        request = kwargs.pop('request', None)
        page_id = kwargs.pop('page_id', None)
        region_name = kwargs.pop('region', None)
        feincms_cls_name = kwargs.pop('cls_name', None)
        self.next_view = kwargs.pop('next_view', None)

        module, cls = feincms_cls_name.split('.')
        ObjectCls = apps.get_model(module, cls)
        feincms_object = ObjectCls.objects.get(id=page_id)

        super(WidgetSelectForm, self).__init__(*args, **kwargs)

        self.fields['page_id'].initial = feincms_object.id
        self.fields['region'].initial = region_name
        self.fields['ordering'].initial = \
            len(getattr(feincms_object.content, region_name, [])) + 1
        self.fields['parent'].initial = feincms_object.id

        choices, grouped, ungrouped = get_grouped_widgets(
            feincms_object, request)

        # reduce choices for validation
        self.fields['cls_name'].choices = [(str(choice[0]), str(choice[1]))
                                           for choice in choices]

        # for now ungrouped to grouped
        grouped['Web'] = ungrouped + grouped.get('Web', [])

        self.helper.layout = Layout(
            Field('region'),
            Field('parent'),
            Field('page_id'),
            Field('ordering'),
            'cls_name',
            Field('first'),
        )

    def handle(self, request, data):
        # NOTE (majklk): This is a bit of a hack, essentially rewriting this
        # request so that we can chain it as an input to the next view...
        # but hey, it totally works.
        request.method = 'GET'

        first = data.pop('first', None)

        if first:
            data['ordering'] = 0

        return self.next_view.as_view()(request, **data)


@memoized
def get_widget_update_form(**kwargs):
    """
    widget = get_widget_from_id(widget_id)

    """
    model_cls = get_class(kwargs['cls_name'])

    form_class_base = getattr(
        model_cls, 'feincms_item_editor_form', WidgetUpdateForm)

    WidgetModelForm = modelform_factory(model_cls,
                                        exclude=[],
                                        form=form_class_base,
                                        widgets=WIDGETS)

    return WidgetModelForm


@memoized
def get_widget_create_form(**kwargs):
    """
    widget = get_widget_from_id(widget_id)

    """
    model_cls = get_class(kwargs['cls_name'])

    form_class_base = getattr(
        model_cls, 'feincms_item_editor_form', WidgetCreateForm)

    WidgetModelForm = modelform_factory(model_cls,
                                        exclude=[],
                                        form=form_class_base,
                                        widgets=WIDGETS)

    return WidgetModelForm



from django.conf.urls import *

urlpatterns = patterns('leonardo.module.media.views',
                       (r'^$', 'directory_list', {
                        'directory_slug': None, 'category_parent_slug': None}, 'leonardo_media_category_simple'),
                       )




from django_select2 import forms
from django import forms as django_forms
from ..models import Folder
from .utils import FOLDER_SEARCH_FIELDS, FileField, FileMultipleField


class FolderPathSelectWidget(forms.Select2Widget):
    search_fields = FOLDER_SEARCH_FIELDS


class FolderSelectWidget(forms.ModelSelect2Widget):
    model = Folder
    search_fields = FOLDER_SEARCH_FIELDS


class FolderFieldMixin(object):

    def label_from_instance(self, obj):
        """
        Coerces ``value`` to a Python data type.
        """
        return obj.pretty_logical_path

    def __init__(self, *args, **kwargs):
        super(FolderFieldMixin, self).__init__(
            queryset=Folder.objects.all(),
            empty_label="---",
            widget=FolderSelectWidget(), *args, **kwargs)


class FolderField(FolderFieldMixin, django_forms.ModelChoiceField):
    '''Folder field for selecting folders, which has human readable label'''
    pass


class FolderMultipleField(FolderFieldMixin, FileMultipleField):
    '''Multiple Select Folder field.'''

    pass




from leonardo.utils.settings import dotdict
from leonardo.utils.settings import get_conf_from_module
from leonardo.utils.settings import get_leonardo_modules
from leonardo.utils.settings import merge
from leonardo.utils.widgets import (find_widget_class, get_all_widget_classes,
                                    get_grouped_widgets)

__all__ = ('dotdict', 'get_conf_from_module', 'get_leonardo_modules',
           'merge', 'find_widget_class', 'get_all_widget_classes',
           'get_grouped_widgets',)


def is_leonardo_module(mod):
    """returns True if is leonardo module
    """

    if hasattr(mod, 'default') \
            or hasattr(mod, 'leonardo_module_conf'):
        return True
    for key in dir(mod):
        if 'LEONARDO' in key:
            return True
    return False




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


default_app_config = 'leonardo.module.search.SearchConfig'


class Default(object):

    optgroup = 'Search'

    @property
    def apps(self):

        INSTALLED_APPS = []
        try:
            import whoosh  # noqa
            INSTALLED_APPS += ['whoosh']
        except Exception:
            pass

        return INSTALLED_APPS + ['haystack', 'leonardo.module.search']

    plugins = [
        ('leonardo.module.search.apps.search', _('Search'))
    ]

class SearchConfig(AppConfig, Default):
    name = 'leonardo.module.search'
    verbose_name = "Search Module"

default = Default()




from optparse import make_option

from django.core.management.base import NoArgsCommand

from ._utils import get_or_create_template

"widget.verbose_name - template.name"
THEME_NAME_FORMAT = "{0} {1}"


class Command(NoArgsCommand):
    help = "Syncs file system templates and themes with the database bidirectionally.\
    based on dbtemplates.sync_templates"
    option_list = NoArgsCommand.option_list + (
        make_option("-e", "--ext",
                    dest="ext", action="store", default="html",
                    help="extension of the files you want to "
                         "sync with the database [default: %default]"),
        make_option("-f", "--force",
                    action="store_true", dest="force", default=False,
                    help="overwrite existing database templates"),
        )

    def handle_noargs(self, **options):
        force = options.get('force')

        get_or_create_template("404.html", force=force, notfix='admin')
        get_or_create_template("403.html", force=force, notfix='admin')
        get_or_create_template("500.html", force=force, notfix='admin')
        get_or_create_template("crossdomain.xml", force=force, extension='.xml')
        get_or_create_template("robots.txt", force=force, extension='.txt')

        self.stdout.write('Successfully synced common templates')



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from leonardo.module.web.models import ListWidget
from leonardo.module.media.fields.folder import FolderField
from leonardo.module.web.widgets.forms import WidgetUpdateForm


class FolderForm(WidgetUpdateForm):

    folder = FolderField()


class DownloadListWidget(ListWidget):

    icon = "fa fa-list-alt"

    feincms_item_editor_form = FolderForm

    folder = models.ForeignKey('media.Folder', verbose_name=_(
        "folder"), related_name="%(app_label)s_%(class)s_folders")

    def get_items(self):
        return self.folder.media_file_files.all()

    class Meta:
        abstract = True
        verbose_name = _("download list")
        verbose_name_plural = _('download lists')




from django.conf.urls import patterns, url

from .views import ModalIframeView

urlpatterns = patterns("",
                       url(r"^modal/(?P<url>.+)/$",
                           ModalIframeView.as_view(), name="modal_iframe"),
                       )




import floppyforms as forms
from django.core import urlresolvers
from django.forms.models import modelformset_factory
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy
from horizon import tables
from horizon.tables.formset import FormsetDataTable, FormsetRow
from leonardo.module.web.models import PageDimension


class Slider(forms.RangeInput):
    min = 1
    max = 12
    step = 1


class OffsetSlider(Slider):
    min = 0


class PageDimensionForm(forms.ModelForm):

    col1_width = forms.CharField(widget=Slider(), initial=4)
    col2_width = forms.CharField(widget=Slider(), initial=4)
    col3_width = forms.CharField(widget=Slider(), initial=4)

    class Meta:
        model = PageDimension
        exclude = tuple()

PageDimensionFormset = modelformset_factory(
    PageDimension, form=PageDimensionForm, can_delete=True, extra=1)


class PageFormsetRow(FormsetRow):

    def __init__(self, column, datum, form):
        self.form = form
        super(PageFormsetRow, self).__init__(column, datum, form)
        # add initial
        if not datum:
            self.form.fields['page'].initial = self.page_object


class PageDimensionAddAction(tables.LinkAction):

    name = "dimension_add"
    verbose_name = _("Override page dimension")
    classes = ("ajax-modal", "btn-edit")
    url = "page_dimension_add"

    def get_link_url(self):
        return urlresolvers.reverse(
            self.url, args=[self.table.page_object.id])

    def allowed(self, request, instance):
        return True


class DeletePageDimension(tables.DeleteAction):

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Page Dimension",
            u"Delete Page Dimension",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Page Dimension",
            u"Deleted Page Dimension",
            count
        )

    def delete(self, request, obj_id):
        from ..models import PageDimension
        PageDimension.objects.get(pk=obj_id).delete()


class PageDimensionTable(tables.DataTable):

    formset_class = PageDimensionFormset

    def get_formset(self):
        if self.page_object:
            queryset = self.page_object.dimensions
        else:
            queryset = PageDimension.objects.none()
        if self._formset is None:
            self._formset = self.formset_class(
                self.request.POST or None,
                initial=self._get_formset_data(),
                prefix=self._meta.name,
                queryset=queryset)
        return self._formset

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):

        #self._meta.row_class = PageFormsetRow
        self.page_object = kwargs.pop('page', None)
        super(PageDimensionTable, self).__init__(
            request, data, needs_form_wrapper, **kwargs)

    page = tables.Column('page')
    size = tables.Column('size', verbose_name=_('Size'))
    col1_width = tables.Column('col1_width', verbose_name=('Column 1 Width'))
    col2_width = tables.Column('col2_width', verbose_name=_('Column 2 Width'))
    col3_width = tables.Column('col3_width', verbose_name=_('Column 3 Width'))

    def get_object_id(self, datum):
        return datum.pk

    def get_object_name(self, datum):
        return str(datum)

    name = 'dimensions'

    class Meta:
        name = 'dimensions'
        table_name = 'Dimensions'
        table_actions = (PageDimensionAddAction,)
        row_actions = (DeletePageDimension,)




import warnings

from leonardo.forms.fields import *

warnings.warn('leonardo.fields is obsolete'
              ' location use leonardo.forms instead'
              'This location is only in migrations now')



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from leonardo.module.web.models import Widget


class UserRegistrationWidget(Widget):

    class Meta:
        abstract = True
        verbose_name = _("user registration")
        verbose_name_plural = _("user registrations")



# -#- coding: utf-8 -#-

from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo.module.web.models import Page, Widget


class PageTitleWidget(Widget):

    class Meta:
        abstract = True
        verbose_name = _("page title")
        verbose_name_plural = _('page titles')

    def render_content(self, options):
        page = self.parent

        try:
            fragments = options['request']._feincms_fragments
        except:
            fragments = {}

        title = fragments.get("_page_title", None)
        subtitle = fragments.get("_page_subtitle", None)

        return render_to_string(self.get_template, {
            'widget': self,
            'request': options['request'],
            'page': page,
            'title': title,
            'subtitle': subtitle,
        })




from django.utils.translation import ugettext_lazy as _

import horizon

from leonardo.module import dashboard


class AuthPanel(horizon.Panel):
    name = _("Auth")
    slug = 'auth'

#dashboard.ModuleDashboard.register(AuthPanel)




from django.conf.urls.defaults import *

urlpatterns = patterns('',
                       url(r'^$', include('haystack.urls'))
                       )

from django.conf.urls.defaults import patterns, url

dict = {
    'template': 'googlesearch/googlesearch_results.html',
    'extra_context': {'title': 'Search Results'}
}

urlpatterns = patterns('django.views.generic.simple',
                       url(r'^$', 'direct_to_template', dict,
                           name='googlesearch_results'),
                       )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import leonardo.module.media.fields.multistorage_file


class Migration(migrations.Migration):

    dependencies = [
        ('media', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='folder',
            options={'ordering': ('name',), 'verbose_name': 'Folder', 'verbose_name_plural': 'Folders', 'permissions': (('can_use_folder_listing', 'Can use Folder listing'),)},
        ),
        migrations.AlterField(
            model_name='file',
            name='file',
            field=leonardo.module.media.fields.multistorage_file.MultiStorageFileField(max_length=255, upload_to=leonardo.module.media.fields.multistorage_file.generate_filename_multistorage, null=True, verbose_name='file', blank=True),
            preserve_default=True,
        ),
    ]



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from leonardo.module.web.models import Widget

ENGINE_CHOICES = (
    ('gooole', _("Google Custom Search")),
    ('haystack', _("Django Haystack")),
)


class SiteSearchWidget(Widget):
    engine = models.CharField(
        max_length=255, verbose_name=_("Search engine"),
        choices=ENGINE_CHOICES, default="google")

    class Meta:
        abstract = True
        verbose_name = _("Site search")
        verbose_name_plural = _("Site searches")




from leonardo.module.nav.mixins import NavigationWidgetMixin
from leonardo.module.nav.models import NavigationWidget
from leonardo.module.web.models import ListWidget, Widget
from leonardo.module.web.widgets.mixins import (AuthContentProxyWidgetMixin,
                                                ContentProxyWidgetMixin,
                                                JSONContentMixin,
                                                ListWidgetMixin)

__all__ = ('Widget', 'ListWidgetMixin', 'ContentProxyWidgetMixin',
           'JSONContentMixin', 'AuthContentProxyWidgetMixin',
           'ListWidget', 'NavigationWidgetMixin', 'NavigationWidget')



from django.apps import apps


def get_widget_from_id(id):
    """returns widget object by id

    example web-htmltextwidget-2-2
    """

    res = id.split('-')
    try:
        model_cls = apps.get_model(res[0], res[1])
        obj = model_cls.objects.get(parent=res[2], id=res[3])
    except:
        obj = None
    return obj


def get_widget_class_from_id(id):
    """returns widget class by id

    example web-htmltextwidget-2-2
    """

    res = id.split('-')
    try:
        model_cls = apps.get_model(res[1], res[2])
    except:
        model_cls = None
    return model_cls




from leonardo.module.web.models import Widget
from leonardo.module.nav.mixins import NavigationWidgetMixin
from feincms.module.page.extensions.navigation import (NavigationExtension,
                                                       PagePretender)
from django.utils.translation import ugettext_lazy as _


class NavigationWidget(Widget, NavigationWidgetMixin):

    """Base class for navigation widgets
    """

    class Meta:
        abstract = True


class FlatPageContentNavigationExtension(NavigationExtension):
    name = _('Flat page content navigation')

    def children(self, page, **kwargs):
        base_url = page.get_absolute_url()
        widget_list = page.objects.filter(parent=None)
        for widget in widget_list:
            subchildren = []
            for subwidget in widget.media_folder_children.all():
                subchildren.append(PagePretender(
                    title=subwidget,
                    url='%s%s/%s/' % (base_url, widget.name, subwidget.name),
                    level=5
                ))
            yield PagePretender(
                title=widget,
                url='%s%s/' % (base_url, widget.name),
                children=subchildren,
                level=5
            )


class NestedPageContentNavigationExtension(NavigationExtension):
    name = _('Nested page content navigation')

    def children(self, page, **kwargs):
        base_url = page.get_absolute_url()
        widget_list = page.objects.filter(parent=None)
        for widget in widget_list:
            subchildren = []
            for subwidget in widget.media_folder_children.all():
                subchildren.append(PagePretender(
                    title=subwidget,
                    url='%s%s/%s/' % (base_url, widget.name, subwidget.name),
                    level=5
                ))
            yield PagePretender(
                title=widget,
                url='%s%s/' % (base_url, widget.name),
                children=subchildren,
                level=5
            )




from __future__ import absolute_import

import os
import six
import logging
import warnings

from django import VERSION
from leonardo.base import leonardo, default
from leonardo.utils.settings import (get_conf_from_module, merge,
                                     get_leonardo_modules, get_loaded_modules,
                                     DJANGO_CONF)
from importlib import import_module  # noqa
from django.utils.module_loading import module_has_submodule  # noqa


_file_path = os.path.abspath(os.path.dirname(__file__)).split('/')

BASE_DIR = '/'.join(_file_path[0:-2])

from leonardo.conf.default import *

MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

MEDIA_URL = '/media/'
STATIC_URL = '/static/'


if VERSION[:2] >= (1, 8):

    TEMPLATES = [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [
                os.path.join(
                    os.path.dirname(os.path.realpath(__file__)), 'templates')
            ],
            'OPTIONS': {
                'context_processors': default.context_processors,
                'loaders': [
                    'django.template.loaders.filesystem.Loader',
                    'django.template.loaders.app_directories.Loader',
                    'dbtemplates.loader.Loader',
                    'horizon.loaders.TemplateLoader',
                ],
                'debug': True
            },
        },
    ]

else:

    TEMPLATE_DIRS = [
        os.path.join(
            os.path.dirname(os.path.realpath(__file__)), 'templates')
    ]

    TEMPLATE_CONTEXT_PROCESSORS = default.context_processors

    TEMPLATE_LOADERS = (
        'dbtemplates.loader.Loader',
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
        'horizon.loaders.TemplateLoader',
    )

try:
    # obsole location since 1.0.3 use `leonrdo_site.settings`
    from leonardo_site.local.settings import *
    warnings.warn(
        'leonardo_site.local.settings is obsolete use new location')
except ImportError:
    pass

try:
    # full settings
    # TODO support configurable from local_settings
    # LEONARDO_PROJECT_NAME = 'leonardo_site'
    from leonardo_site.settings import *
except ImportError:
    pass

try:
    # local settings
    from local_settings import *
except ImportError:
    warnings.warn(
        'local_settings was not found in $PYTHONPATH !')

if not DEBUG:
    if VERSION[:2] >= (1, 8):
        TEMPLATES[0]['OPTIONS']['loaders'] = [
            ('django.template.loaders.cached.Loader', [
                'dbtemplates.loader.Loader',
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
                'horizon.loaders.TemplateLoader',
            ])]
        TEMPLATES[0]['OPTIONS']['debug'] = False
else:
    TEMPLATE_DEBUG = DEBUG

APPS = merge(APPS, default.core)

if 'media' in APPS:
    FILER_IMAGE_MODEL = 'leonardo.module.media.models.Image'

try:
    from leonardo.conf.horizon import *
    from leonardo.conf.static import *
except Exception as e:
    pass

if LEONARDO_SYSTEM_MODULE:
    APPS = merge(APPS, ['leonardo_system'])
    HORIZON_CONFIG['system_module'] = True
else:
    HORIZON_CONFIG['system_module'] = False

# load directly specified apps
leonardo.get_app_modules(APPS)

# propagate settings to leonardo
leonardo.MODULES_AUTOLOAD = LEONARDO_MODULE_AUTO_INCLUDE

# load all modules
leonardo.load_modules()

# just propagate all loaded modules to settings
LEONARDO_MODULES = leonardo.get_modules()

# iterate over sorted modules
for mod, mod_cfg in LEONARDO_MODULES:

    try:

        # import all settings keys from module
        if module_has_submodule(mod, "settings"):
            try:
                settings_mod = import_module(
                    '{0}.settings'.format(mod.__name__))
                for k in dir(settings_mod):
                    if not k.startswith("_"):
                        val = getattr(settings_mod, k, None)
                        globals()[k] = val
                        locals()[k] = val
            except Exception as e:
                warnings.warn(
                    'Exception "{}" raised during loading '
                    'settings from {}'.format(str(e), mod))

        # go through django keys and merge it to main settings
        for key in DJANGO_CONF.keys():
            updated_value = mod_cfg.get_value(key, globals()[key])
            globals()[key] = updated_value
            locals()[key] = updated_value
            # map value to leonardo but under our internal name
            setattr(leonardo, DJANGO_CONF[key], updated_value)

        if mod_cfg.urls_conf:
            MODULE_URLS[mod_cfg.urls_conf] = {'is_public': mod_cfg.public}
        # TODO move to utils.settings
        # support for one level nested in config dictionary
        for config_key, config_value in six.iteritems(mod_cfg.config):

            if isinstance(config_value, dict):
                CONSTANCE_CONFIG_GROUPS.update({config_key: config_value})
                for c_key, c_value in six.iteritems(config_value):
                    mod_cfg.config[c_key] = c_value
                # remove from main dict
                mod_cfg.config.pop(config_key)
            else:
                if isinstance(mod_cfg.optgroup, six.string_types):
                    CONSTANCE_CONFIG_GROUPS.update({
                        mod_cfg.optgroup: mod_cfg.config})
                else:
                    if 'ungrouped' in CONSTANCE_CONFIG_GROUPS:
                        CONSTANCE_CONFIG_GROUPS['ungrouped'].update(mod_cfg.config)
                    else:
                        CONSTANCE_CONFIG_GROUPS['ungrouped'] = \
                            mod_cfg.config

        # import and update absolute overrides
        for model, method in six.iteritems(mod_cfg.absolute_url_overrides):
            try:
                _mod = import_module(".".join(method.split('.')[:-1]))
                ABSOLUTE_URL_OVERRIDES[model] = getattr(
                    _mod, method.split('.')[-1])
            except Exception as e:
                raise e

        for nav_extension in mod_cfg.navigation_extensions:
            try:
                import_module(nav_extension)
            except ImportError:
                pass

        CONSTANCE_CONFIG.update(mod_cfg.config)

        if VERSION[:2] >= (1, 8):
            TEMPLATES[0]['DIRS'] = merge(TEMPLATES[0]['DIRS'], mod_cfg.dirs)
            cp = TEMPLATES[0]['OPTIONS']['context_processors']
            TEMPLATES[0]['OPTIONS']['context_processors'] = merge(
                cp, mod_cfg.context_processors)

        else:

            TEMPLATE_CONTEXT_PROCESSORS = merge(
                TEMPLATE_CONTEXT_PROCESSORS, mod_cfg.context_processors)
            TEMPLATE_DIRS = merge(TEMPLATE_DIRS, mod_cfg.dirs)

        # collect grouped widgets
        if isinstance(mod_cfg.optgroup, six.string_types):
            if len(mod_cfg.widgets) > 0:
                WIDGETS[mod_cfg.optgroup] = merge(
                    getattr(WIDGETS, mod_cfg.optgroup, []), mod_cfg.widgets)
        else:
            if len(mod_cfg.widgets) > 0 and DEBUG:
                WIDGETS['ungrouped'] = merge(
                    getattr(WIDGETS, 'ungrouped', []), mod_cfg.widgets)
                warnings.warn('You have ungrouped widgets'
                              ', please specify your ``optgroup``'
                              'which categorize your widgets in %s' % mod)

    except Exception as e:
        warnings.warn(
            'Exception "{}" raised during loading '
            'module {}'.format(str(e), mod))

setattr(leonardo, 'widgets', WIDGETS)

from leonardo.module.web.models import Page
from leonardo.module.web.widget import ApplicationWidget

# register external apps
Page.create_content_type(
    ApplicationWidget, APPLICATIONS=APPLICATION_CHOICES)

# register widgets
for _optgroup, _widgets in six.iteritems(WIDGETS):
    optgroup = _optgroup if _optgroup != 'ungrouped' else None
    for widget in _widgets:
        Page.create_content_type(widget, optgroup=optgroup)

Page.register_extensions(*PAGE_EXTENSIONS)
Page.register_default_processors(LEONARDO_FRONTEND_EDITING)

# FINALLY OVERRIDE ALL

try:
    # local settings
    from local_settings import *
except ImportError:
    warnings.warn(
        'Missing local_settings !')

try:
    # full settings
    from leonardo_site.local.settings import *
except ImportError:
    pass

try:
    # full settings
    from leonardo_site.settings import *
except ImportError:
    pass

# and again merge core with others
APPS = merge(APPS, default.core)

# go through django keys and merge it to main settings
for key in DJANGO_CONF.keys():
    # map value to leonardo but under our internal name
    setattr(leonardo, DJANGO_CONF[key], globals()[key])

# Add HORIZON_CONFIG to the context information for offline compression
COMPRESS_OFFLINE_CONTEXT = {
    'STATIC_URL': STATIC_URL,
    'HORIZON_CONFIG': HORIZON_CONFIG,
}

if DEBUG:

    try:
        import debug_toolbar
        INSTALLED_APPS = merge(INSTALLED_APPS, ['debug_toolbar'])
        from leonardo.conf.debug import *
    except ImportError:
        if DEBUG:
            warnings.warn('DEBUG is set to True but, DEBUG tools '
                          'is not installed please run '
                          '"pip install django-leonardo[debug]"')

# async messages
try:
    import async_messages
    INSTALLED_APPS = merge(INSTALLED_APPS, ['async_messages'])
    MIDDLEWARE_CLASSES = merge(MIDDLEWARE_CLASSES,
                               ['async_messages.middleware.AsyncMiddleware'])
except ImportError:
    pass

# use js files instead of horizon
HORIZON_CONFIG['js_files'] = leonardo.js_files
HORIZON_CONFIG['js_spec_files'] = leonardo.js_spec_files
HORIZON_CONFIG['css_files'] = leonardo.css_files
HORIZON_CONFIG['scss_files'] = leonardo.scss_files
HORIZON_CONFIG['angular_modules'] = leonardo.angular_modules
HORIZON_CONFIG['page_actions'] = leonardo.page_actions
HORIZON_CONFIG['widget_actions'] = leonardo.widget_actions
# path horizon config
from horizon import conf
conf.HORIZON_CONFIG = HORIZON_CONFIG

if DEBUG:
    logging.basicConfig(level=logging.DEBUG)




from .icon.models import IconWidget
from .application.models import ApplicationWidget
from .markuptext.models import MarkupTextWidget
from .feedreader.models import FeedReaderWidget
from .pagetitle.models import PageTitleWidget
from .table.models import TableWidget
from .siteheading.models import SiteHeadingWidget
from .htmltext.models import HtmlTextWidget





from __future__ import unicode_literals

import json

from django.db import models
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from .const import PAGINATION_CHOICES

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


class ListWidgetMixin(models.Model):

    """Provide API for listing items
    """

    objects_per_page = models.PositiveIntegerField(
        verbose_name=_('Objects per page'), blank=True, default=6)

    objects_per_row = models.PositiveIntegerField(
        verbose_name=_('Objects per row'), blank=True, default=3)

    pagination_style = models.CharField(
        verbose_name=_("Pagination Style"), max_length=50,
        choices=PAGINATION_CHOICES, default='paginator')

    def get_items(self, request=None):
        '''returns queryset or array of items for listing'''
        raise NotImplementedError

    def filter_items(self, items):
        '''perform filtering items by specific criteria'''
        return items

    def set_items(self, items):
        '''just setter for items'''
        self._items = items

    @cached_property
    def items(self):
        '''access for filtered items'''
        if hasattr(self, '_items'):
            return self.filter_items(self._items)
        self._items = self.get_items()
        return self.filter_items(self._items)

    def populate_items(self, request):
        '''populate and returns filtered items'''
        self._items = self.get_items(request)
        return self.items

    @cached_property
    def get_list_template(self):
        '''returns base list template by pagination_style'''
        return "base/widget/list/_%s.html" % self.pagination_style

    @cached_property
    def get_rows(self):
        '''returns rows with items
        [[item1 item2 item3], [item2 ]]'''
        rows = []
        row = []
        for i, item in enumerate(self.items):
            if self.objects_per_row == i:
                rows.append(row)
                row = []
                i = 0
            row.append(item)
        rows.append(row)
        return rows

    @cached_property
    def columns_classes(self):
        '''returns columns count'''
        md = 12 / self.objects_per_row
        sm = None
        if self.objects_per_row > 2:
            sm = 12 / (self.objects_per_row / 2)
        return md, (sm or md), 12

    @cached_property
    def get_pages(self):
        '''returns pages with rows'''
        pages = []
        page = []
        for i, item in enumerate(self.get_rows):
            if self.objects_per_page == i:
                pages.append(page)
                page = []
                i = 0
            page.append(item)
        pages.append(page)
        return pages

    @cached_property
    def needs_pagination(self):
        if self.objects_per_page == 0:
            return False
        if len(self.items) > self.objects_per_page \
                or len(self.get_pages[0]) >= self.objects_per_page:
            return True
        return False

    @cached_property
    def get_item_template(self):
        '''returns template for one item from queryset'''
        return "widget/%s/_item.html" % self.widget_name

    def __init__(self, *args, **kwargs):
        super(ListWidgetMixin, self).__init__(*args, **kwargs)

        get_items = getattr(self, 'get_items', None)
        render = getattr(self, 'render', None)
        if not callable(get_items) or not callable(render):
            raise Exception('bases on ListWidgetMixin must '
                            'have implemented get_items or render method')

    class Meta:
        abstract = True


class ContentProxyWidgetMixin(models.Model):

    """Provide basic fields and routines
    for loading and caching data from external resource

    define your implementation for getting data in ``get_data``
    and use ``data`` property in your templates
    """

    source_address = models.CharField(
        verbose_name=_("Source Address"), max_length=255)

    cache_validity = models.PositiveIntegerField(
        verbose_name=_('Cache validity'), default=3600)

    cache_data = models.TextField(
        verbose_name=_("Cache data"), blank=True)

    cache_updated = models.DateTimeField(
        verbose_name=_('Cache update'),
        editable=False, null=True, blank=True)

    @cached_property
    def address_parser(self):
        return urlparse(self.source_address)

    @cached_property
    def get_port(self):
        """returns parsed port from ``source_address``
        """
        return self.address_parser.port

    @cached_property
    def get_host(self):
        """returns parsed host from ``source_address``
        """
        return self.address_parser.hostname

    def is_obsolete(self):
        """returns True is data is obsolete and needs revalidation
        """
        if self.cache_updated:
            now = timezone.now()
            delta = now - self.cache_updated
            if delta.seconds < self.cache_validity:
                return False
        return True

    def update_cache(self, data=None):
        """call with new data or set data to self.cache_data and call this
        """
        if data:
            self.cache_data = data
        self.cache_updated = timezone.now()
        self.save()

    def get_data(self, *args, **kwargs):
        """define your behavior for loading raw data
        """
        raise NotImplementedError

    @property
    def data(self):
        """this property just calls ``get_data``
        but here you can serilalize your data or render as html
        these data will be saved to self.cached_content
        also will be accessable from template
        """
        if self.is_obsolete():
            self.update_cache(self.get_data())
        return self.cache_data

    class Meta:
        abstract = True


class JSONContentMixin(object):

    """just expect json data from ``get_data`` method
    """

    @property
    def data(self):
        """load and cache data in json format
        """

        if self.is_obsolete():
            self.cache_data = json.dumps(self.get_data())
            self.update_cache()
        return json.loads(self.cache_data)


class AuthContentProxyWidgetMixin(models.Model):

    """widget mixin for getting remote content with credentials
    """

    username = models.CharField(
        verbose_name=_("Username"), max_length=255, blank=True, null=True)

    password = models.CharField(
        verbose_name=_('Password'), max_length=255, blank=True, null=True)

    token = models.CharField(
        verbose_name=_('API Token'), max_length=255, blank=True, null=True)

    class Meta:
        abstract = True



# -#- coding: utf-8 -#-

from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo.module.nav.forms import NavigationForm
from leonardo.module.nav.models import NavigationWidget
from leonardo.module.web.const import PAGE_REGIONS
from leonardo.module.web.models import Page
from leonardo.module.web.widgets.mixins import ListWidgetMixin

DEPTH_CHOICES = (
    (0, _("self")),
    (1, _("one level")),
    (2, _("all levels")),
)

LINK_CHOICES = (
    ('', _("none")),
    ('link', _("link")),
    ('button', _("button")),
)

class ContextNavigationWidget(ListWidgetMixin, NavigationWidget):
    root = models.ForeignKey(Page, blank=True, null=True, verbose_name=_("Source page"), related_name="context_root", help_text=_('The child pages of root page are displayed in the context navigation.'))
    page_region = models.CharField(max_length=255, verbose_name=_("Region to display"), choices=PAGE_REGIONS, default='preview', help_text=_('Which region of selected pages do you wish to display.'))
    depth = models.IntegerField(verbose_name=_("Depth"), choices=DEPTH_CHOICES, default=1, help_text=_('Depth to which display child pages.'))
    link_style = models.CharField(max_length=255, verbose_name=_("Link style"), choices=LINK_CHOICES, default='link', blank=True, help_text=_('Visual style of links to displayed pages.'))
    link_text = models.CharField(max_length=255, verbose_name=_("Link label"), default='', blank=True, help_text=_('Arbitrary text of links. If empty, page\'s title will be used instead.'))

    class Meta:
        abstract = True
        verbose_name = _("Linked page")
        verbose_name_plural = _('Linked pages')

    feincms_item_editor_form = NavigationForm

    def render(self, **kwargs):
        return self.render_with_cache(kwargs)

    def filter_items(self, items):
        return super(NavigationWidget, self).filter_items(items)

    def get_items(self, request=None):
        '''returns queryset or array of items for listing'''
        return self.get_root(request).children.all()

    def get_root(self, request=None):

        if self.root:
            root = self.root
        else:
            if not request:
                raise Exception('call populate_items with request before access to data')
            root = request.leonardo_page
        return root

    def render_content(self, options):

        self.populate_items(options['request'])

        return render_to_string(self.get_template_name(), { 
            'widget': self,
            'page': self.get_root(options['request']),
            'items': self.items,
            'request': options['request'],
            'region': self.page_region,
        })



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.utils.translation import get_language_from_request
from leonardo.module.nav.forms import NavigationForm
from django.utils.functional import cached_property

from leonardo.module.web.models import Widget, Page


class SiteMapWidget(Widget):

    feincms_item_editor_form = NavigationForm

    root = models.ForeignKey(Page, blank=True, null=True, verbose_name=_(
        "Root page"), related_name="sitemap_root", help_text=_("If no root page is set, widget's parent page will be used."))

    class Meta:
        abstract = True
        verbose_name = _("Site map")
        verbose_name_plural = _('Site maps')

    @cached_property
    def get_root(self):
        return self.root if self.root else self.parent

    def render_content(self, options):
        request = options['request']
        page_list = self.root.get_children()

        return render_to_string(self.get_template_name(), {
            'widget': self,
            'page_list': page_list,
            'request': request,
        })



"""
Third-party application inclusion support.
"""

from __future__ import absolute_import, unicode_literals

from random import SystemRandom

import six
from django.core.cache import cache
from django.core.urlresolvers import *
from django.core.urlresolvers import get_script_prefix, set_script_prefix
from django.template.response import TemplateResponse

from .models import ApplicationWidget

APP_REVERSE_CACHE_GENERATION_KEY = 'FEINCMS:APPREVERSECACHE'
APP_REVERSE_CACHE_TIMEOUT = 300


class UnpackTemplateResponse(TemplateResponse):

    """
    Completely the same as marking applicationcontent-contained views with
    the ``feincms.views.decorators.unpack`` decorator.
    """
    _feincms_unpack = True


def cycle_app_reverse_cache(*args, **kwargs):
    """Does not really empty the cache; instead it adds a random element to the
    cache key generation which guarantees that the cache does not yet contain
    values for all newly generated keys"""
    value = '%07x' % (SystemRandom().randint(0, 0x10000000))
    cache.set(APP_REVERSE_CACHE_GENERATION_KEY, value)
    return value


# Set the app_reverse_cache_generation value once per startup (at least).
# This protects us against offline modifications of the database.
cycle_app_reverse_cache()


def app_reverse(viewname, urlconf=None, args=None, kwargs=None,
                *vargs, **vkwargs):
    """
    Reverse URLs from application contents
    Works almost like Django's own reverse() method except that it resolves
    URLs from application contents. The second argument, ``urlconf``, has to
    correspond to the URLconf parameter passed in the ``APPLICATIONS`` list
    to ``Page.create_content_type``::
        app_reverse('mymodel-detail', 'myapp.urls', args=...)
        or
        app_reverse('mymodel-detail', 'myapp.urls', kwargs=...)
    The second argument may also be a request object if you want to reverse
    an URL belonging to the current application content.
    """

    # First parameter might be a request instead of an urlconf path, so
    # we'll try to be helpful and extract the current urlconf from it
    extra_context = getattr(urlconf, '_feincms_extra_context', {})
    appconfig = extra_context.get('app_config', {})
    urlconf = appconfig.get('urlconf_path', urlconf)
    appcontent_class = ApplicationWidget._feincms_content_models[0]
    cache_key = appcontent_class.app_reverse_cache_key(urlconf)
    url_prefix = cache.get(cache_key)

    if url_prefix is None:
        content = appcontent_class.closest_match(urlconf)

        if content is not None:
            if urlconf in appcontent_class.ALL_APPS_CONFIG:
                # We have an overridden URLconf
                app_config = appcontent_class.ALL_APPS_CONFIG[urlconf]
                urlconf = app_config['config'].get('urls', urlconf)

            prefix = content.parent.get_absolute_url()
            prefix += '/' if prefix[-1] != '/' else ''

            url_prefix = (urlconf, prefix)
            cache.set(cache_key, url_prefix, timeout=APP_REVERSE_CACHE_TIMEOUT)

    if url_prefix:
        # vargs and vkwargs are used to send through additional parameters
        # which are uninteresting to us (such as current_app)
        prefix = get_script_prefix()
        try:
            set_script_prefix(url_prefix[1])
            return reverse(
                viewname,
                url_prefix[0],
                args=args,
                kwargs=kwargs,
                *vargs, **vkwargs)
        finally:
            set_script_prefix(prefix)

    raise NoReverseMatch("Unable to find ApplicationContent for %r" % urlconf)


#: Lazy version of ``app_reverse``
app_reverse_lazy = lazy(app_reverse, str)


def permalink(func):
    """
    Decorator that calls app_reverse()
    Use this instead of standard django.db.models.permalink if you want to
    integrate the model through ApplicationContent. The wrapped function
    must return 4 instead of 3 arguments::
        class MyModel(models.Model):
            @appmodels.permalink
            def get_absolute_url(self):
                return ('myapp.urls', 'model_detail', (), {'slug': self.slug})
    """
    def inner(*args, **kwargs):
        return app_reverse(*func(*args, **kwargs))
    return wraps(func)(inner)


def reverse(viewname, urlconf=None, args=None, kwargs=None, current_app=None):
    """monkey patched reverse

    path supports easy patching 3rd party urls
    if 3rd party app has namespace for example ``catalogue`` and
    you create FeinCMS plugin with same name as this namespace reverse
    returns url from ApplicationContent !

    """

    if not urlconf:
        urlconf = get_urlconf()
    resolver = get_resolver(urlconf)
    args = args or []
    kwargs = kwargs or {}

    prefix = get_script_prefix()

    if not isinstance(viewname, six.string_types):
        view = viewname
    else:
        parts = viewname.split(':')
        parts.reverse()
        view = parts[0]
        path = parts[1:]

        resolved_path = []
        ns_pattern = ''
        while path:
            ns = path.pop()

            # Lookup the name to see if it could be an app identifier
            try:
                app_list = resolver.app_dict[ns]
                # Yes! Path part matches an app in the current Resolver
                if current_app and current_app in app_list:
                    # If we are reversing for a particular app,
                    # use that namespace
                    ns = current_app
                elif ns not in app_list:
                    # The name isn't shared by one of the instances
                    # (i.e., the default) so just pick the first instance
                    # as the default.
                    ns = app_list[0]
            except KeyError:
                pass

            try:
                extra, resolver = resolver.namespace_dict[ns]
                resolved_path.append(ns)
                ns_pattern = ns_pattern + extra
            except KeyError as key:
                for urlconf, config in six.iteritems(
                        ApplicationWidget._feincms_content_models[0].ALL_APPS_CONFIG):
                    partials = viewname.split(':')[1:]

                    try:
                        url = app_reverse(
                            ':'.join(partials), urlconf, args=args, kwargs=kwargs,
                            current_app=current_app)
                    except NoReverseMatch:
                        pass
                    else:
                        # ensure that viewname is in urlconf
                        if urlconf.split(".")[-1] in viewname:
                            return url

                if resolved_path:
                    raise NoReverseMatch(
                        "%s is not a registered namespace inside '%s'" %
                        (key, ':'.join(resolved_path)))
                else:
                    raise NoReverseMatch("%s is not a registered namespace" %
                                         key)
        if ns_pattern:
            resolver = get_ns_resolver(ns_pattern, resolver)

    return iri_to_uri(resolver._reverse_with_prefix(view, prefix, *args, **kwargs))


reverse_lazy = lazy(reverse, six.text_type)




from __future__ import absolute_import

from django.utils.translation import ugettext_lazy as _
from leonardo.views import *
from leonardo import messages

from leonardo.module.web.models import Page

from .forms import PageMassChangeForm


class PageObjectMixin(object):

    @property
    def object(self):

        try:
            obj = Page.objects.get(id=self.kwargs["page_id"])
        except Exception as e:
            messages.error(self.request, str(e))
        return obj


class PageMassUpdateView(PageObjectMixin, ModalFormView):

    """Page Mass Update of Theme, Color Scheme and Layout
    with custom depth
    """

    form_class = PageMassChangeForm

    def get_context_data(self, **kwargs):
        context = super(PageMassUpdateView, self).get_context_data(**kwargs)
        # add extra context for template
        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = _("Change Pages")
        context['form_submit'] = _("Apply")
        return context

    def form_invalid(self, form):
        raise Exception(form.errors)

    def get_initial(self):
        return {'page_id': self.kwargs["page_id"]}



from __future__ import unicode_literals

from optparse import make_option

from django.core.management.base import BaseCommand, NoArgsCommand
from .sync_page_themes import Command as PageThemeCommand
from .sync_widget_themes import Command as SyncWidgetsCommand


class Command(BaseCommand):

    help = "Syncs templates and themes"
    option_list = NoArgsCommand.option_list + (
        make_option("-f", "--force",
                    action="store_true", dest="force", default=False,
                    help="overwrite existing database templates"),
        make_option('--noinput',
                    action='store_false', dest='interactive', default=True,
                    help="Do NOT prompt the user for input of any kind."),
    )

    def handle(self, **options):
        force = options.get('force', False)

        # sync widgets
        sync_cmd = SyncWidgetsCommand()
        sync_cmd.stdout = self.stdout
        sync_cmd.handle(**{
            'force': force,
            'verbosity': False})

        # sync page themes
        sync_cmd = PageThemeCommand()
        sync_cmd.stdout = self.stdout
        sync_cmd.handle(**{
            'force': force,
            'verbosity': False})




from __future__ import absolute_import

from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from leonardo.views import *
from leonardo import messages

from ..models import Page
from .dimension.forms import PageDimensionForm
from .tables import PageDimensionTable
from .forms import PageCreateForm, PageDeleteForm, PageUpdateForm


class PageCreateView(ModalFormView):

    form_class = PageCreateForm

    @property
    def parent(self):

        try:
            obj = Page.objects.get(id=self.kwargs["parent"])
        except Exception as e:
            raise e
        return obj

    def get_context_data(self, **kwargs):
        context = super(PageCreateView, self).get_context_data(**kwargs)
        # add extra context for template
        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = _("Create Page")
        context['title'] = "self.get_header()"
        context['form_submit'] = _("Create")
        context['heading'] = "self.get_header()"
        context['modal_size'] = "lg"
        return context

    def form_valid(self, form):
        try:
            page = form.save()
            messages.success(
                self.request, _('Page %s was successfully crated.' % page))
        except Exception as e:
            messages.error(self.request, str(e))
            return HttpResponseRedirect(self.object.get_absolute_url())

        return HttpResponseRedirect(page.get_absolute_url())

    def get_initial(self):
        parent = self.parent
        initial = {
            'parent': parent,
            'slug': self.kwargs.get('slug', None),
            'color_scheme': parent.color_scheme,
            'theme': parent.theme,
            'layout': parent.layout,
            'site': parent.site,
            'template_key': parent.template_key,
            'in_navigation': parent.in_navigation,
        }
        if 'slug' in self.kwargs:
            initial['slug'] = self.kwargs['slug']
            initial['title'] = self.kwargs['slug'].capitalize()
        return initial


class PageUpdateView(ModalFormView):

    form_class = PageUpdateForm

    @property
    def object(self):

        try:
            obj = Page.objects.get(id=self.kwargs["page_id"])
        except Exception as e:
            messages.error(self.request, str(e))
        return obj

    def get_context_data(self, **kwargs):
        context = super(PageUpdateView, self).get_context_data(**kwargs)
        # add extra context for template
        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = _("Update Page")
        context['title'] = "self.get_header()"
        context['form_submit'] = _("Update")
        context['heading'] = "self.get_header()"
        context['modal_size'] = "lg"
        return context

    def get_form(self, form_class):
        kwargs = self.get_form_kwargs()
        kwargs.update({
            'request': self.request,
            'instance': self.object,
            'initial': {'page': self.object}
        })
        return self.form_class(**kwargs)

    def form_valid(self, form):
        try:
            page = form.save()
        except Exception as e:
            messages.error(self.request, str(e))

        return HttpResponseRedirect(page.get_absolute_url())

    def form_invalid(self, form):
        raise Exception(form.errors)

    def construct_tables(self):
        table = PageDimensionTable(
            self.request, data=self.object.dimensions.all())
        # Early out before data is loaded
        preempted = table.maybe_preempt()
        if preempted:
            return preempted

        # handle actions
        handled = table.maybe_handle()
        if handled:
            return handled

        # If we didn't already return a response, returning None continues
        # with the view as normal.
        return None

    def post(self, request, *args, **kwargs):
        # GET and POST handling are the same
        handled = self.construct_tables()
        if handled:
            return handled
        return super(PageUpdateView, self).post(request, *args, **kwargs)


class PageDimensionUpdateView(ModalFormView):

    form_class = PageDimensionForm

    @property
    def page(self):

        try:
            obj = Page.objects.get(id=self.kwargs["page_id"])
        except Exception as e:
            raise e
        return obj

    def get_context_data(self, **kwargs):
        context = super(PageDimensionUpdateView, self).get_context_data(**kwargs)
        # add extra context for template
        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = _("Add Page Dimesion")
        context['title'] = "self.get_header()"
        context['view_name'] = _("Create")
        context['heading'] = "self.get_header()"
        return context

    def get_initial(self):
        return {'page': self.page}

    def form_valid(self, form):
        try:
            dimension = form.save()
        except Exception as e:
            messages.error(self.request, str(e))

        return HttpResponseRedirect(dimension.page.get_absolute_url())


class PageDeleteView(ModalFormView, ContextMixin, ModelFormMixin):

    form_class = PageDeleteForm

    @property
    def object(self):

        try:
            obj = Page.objects.get(id=self.kwargs["page_id"])
        except Exception as e:
            raise e
        return obj

    def get_context_data(self, **kwargs):
        context = super(PageDeleteView, self).get_context_data(**kwargs)

        page = self.object

        # add extra context for template
        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = 'Delete {} ?'.format(page)
        context['title'] = 'Delete {}'.format(page)
        context['form_submit'] = _('Delete')
        context['heading'] = 'Delete {} ?'.format(page)
        context['help_text'] = self.get_help_text()
        return context

    def form_valid(self, form):
        obj = self.object
        try:
            parent = obj.parent
            obj.delete()
            success_url = parent.get_absolute_url()
            response = HttpResponseRedirect(success_url)
            response['X-Horizon-Location'] = success_url
        except Exception as e:
            raise e

        return response

    def get_initial(self):
        return self.kwargs




import base64
import hashlib
import warnings
from io import BytesIO

from django.core.files.base import ContentFile
from django.utils import six

from easy_thumbnails import (fields as easy_thumbnails_fields,
                             files as easy_thumbnails_files)

from .. import settings as filer_settings
from filer.utils.filer_easy_thumbnails import ThumbnailerNameMixin

from django.conf import settings

STORAGES = {
    'public': filer_settings.FILER_PUBLICMEDIA_STORAGE,
    'private': filer_settings.FILER_PRIVATEMEDIA_STORAGE,
}
THUMBNAIL_STORAGES = {
    'public': filer_settings.FILER_PUBLICMEDIA_THUMBNAIL_STORAGE,
    'private': filer_settings.FILER_PRIVATEMEDIA_THUMBNAIL_STORAGE,
}
THUMBNAIL_OPTIONS = {
    'public': filer_settings.FILER_PUBLICMEDIA_THUMBNAIL_OPTIONS,
    'private': filer_settings.FILER_PRIVATEMEDIA_THUMBNAIL_OPTIONS,
}


def generate_filename_multistorage(instance, filename):
    if instance.is_public:
        upload_to = settings.FILER_PUBLICMEDIA_UPLOAD_TO
    else:
        upload_to = settings.FILER_PRIVATEMEDIA_UPLOAD_TO

    if callable(upload_to):
        return upload_to(instance, filename)
    else:
        return upload_to


class MultiStorageFieldFile(ThumbnailerNameMixin,
                            easy_thumbnails_files.ThumbnailerFieldFile):
    def __init__(self, instance, field, name):
        """
        This is a little weird, but I couldn't find a better solution.
        Thumbnailer.__init__ is called first for proper object inizialization.
        Then we override some attributes defined at runtime with properties.
        We cannot simply call super().__init__ because filer Field objects
        doesn't have a storage attribute.
        """
        easy_thumbnails_files.Thumbnailer.__init__(self, None, name)
        self.instance = instance
        self.field = field
        self._committed = True
        self.storages = self.field.storages
        self.thumbnail_storages = self.field.thumbnail_storages
        self.thumbnail_options = self.field.thumbnail_options
        self.storage = self._storage
        self.source_storage = self._source_storage
        self.thumbnail_storage = self._thumbnail_storage
        self.thumbnail_basedir = self._thumbnail_base_dir

    @property
    def _storage(self):
        if self.instance.is_public:
            return self.storages['public']
        else:
            return self.storages['private']

    @property
    def _source_storage(self):
        if self.instance.is_public:
            return self.storages['public']
        else:
            return self.storages['private']

    @property
    def _thumbnail_storage(self):
        if self.instance.is_public:
            return self.thumbnail_storages['public']
        else:
            return self.thumbnail_storages['private']

    @property
    def _thumbnail_base_dir(self):
        if self.instance.is_public:
            return self.thumbnail_options['public'].get('base_dir', '')
        else:
            return self.thumbnail_options['private'].get('base_dir', '')

    def save(self, name, content, save=True):
        content.seek(0) # Ensure we upload the whole file
        super(MultiStorageFieldFile, self).save(name, content, save)


class MultiStorageFileField(easy_thumbnails_fields.ThumbnailerField):
    attr_class = MultiStorageFieldFile

    def __init__(self, verbose_name=None, name=None,
                 storages=None, thumbnail_storages=None, thumbnail_options=None, **kwargs):
        if 'upload_to' in kwargs:
            upload_to = kwargs.pop("upload_to")
            if upload_to != generate_filename_multistorage:
                warnings.warn("MultiStorageFileField can handle only File objects;"
                              "%s passed" % upload_to, SyntaxWarning)
        self.storages = storages or STORAGES
        self.thumbnail_storages = thumbnail_storages or THUMBNAIL_STORAGES
        self.thumbnail_options = thumbnail_options or THUMBNAIL_OPTIONS
        super(easy_thumbnails_fields.ThumbnailerField, self).__init__(
                                      verbose_name=verbose_name, name=name,
                                      upload_to=generate_filename_multistorage,
                                      storage=None, **kwargs)

    def value_to_string(self, obj):
        value = super(MultiStorageFileField, self).value_to_string(obj)
        if not filer_settings.FILER_DUMP_PAYLOAD:
            return value
        try:
            payload_file = BytesIO(self.storage.open(value).read())
            sha = hashlib.sha1()
            sha.update(payload_file.read())
            if sha.hexdigest() != obj.sha1:
                warnings.warn('The checksum for "%s" diverges. Check for file consistency!' % obj.original_filename)
            payload_file.seek(0)
            encoded_string = base64.b64encode(payload_file.read()).decode('utf-8')
            return value, encoded_string
        except IOError:
            warnings.warn('The payload for "%s" is missing. No such file on disk: %s!' % (obj.original_filename, self.storage.location))
            return value

    def to_python(self, value):
        if isinstance(value, list) and len(value) == 2 and isinstance(value[0], six.text_type):
            filename, payload = value
            try:
                payload = base64.b64decode(payload)
            except TypeError:
                pass
            else:
                if self.storage.exists(filename):
                    self.storage.delete(filename)
                self.storage.save(filename, ContentFile(payload))
                return filename
        return value




from django.contrib import admin
from django.core.urlresolvers import reverse
from filer.utils.compatibility import DJANGO_1_7


class PrimitivePermissionAwareModelAdmin(admin.ModelAdmin):

    def has_add_permission(self, request):
        # we don't have a "add" permission... but all adding is handled
        # by special methods that go around these permissions anyway
        # TODO: reactivate return False
        return True

    def has_change_permission(self, request, obj=None):
        if hasattr(obj, 'has_edit_permission'):
            if obj.has_edit_permission(request):
                return True
            else:
                return False
        else:
            return True

    def has_delete_permission(self, request, obj=None):
        # we don't have a specific delete permission... so we use change
        return self.has_change_permission(request, obj)

    def _get_post_url(self, obj):
        """ Needed to retrieve the changelist url as Folder/File can be extended
        and admin url may change """
        # Code borrowed from django ModelAdmin to determine changelist on the fly
        opts = obj._meta
        if DJANGO_1_7:
            model_name = opts.module_name
        else:
            model_name = opts.model_name
        return reverse('admin:%s_%s_changelist' %
                       (opts.app_label, model_name),
                       current_app=self.admin_site.name)




import copy

import floppyforms as forms
from crispy_forms.bootstrap import Tab, TabHolder
from crispy_forms.layout import Field, HTML, Layout
from django.utils.translation import ugettext_lazy as _
from horizon.utils.memoized import memoized
from horizon_contrib.forms import SelfHandlingModelForm

from leonardo.module.web.models import PageDimension


class Slider(forms.RangeInput):
    min = 0
    max = 12
    step = 1


class PageDimensionForm(SelfHandlingModelForm):

    col1_width = forms.CharField(widget=Slider(), initial=4)
    col2_width = forms.CharField(widget=Slider(), initial=4)
    col3_width = forms.CharField(widget=Slider(), initial=4)

    def _check_dimensions(self):
        col1_width = self.cleaned_data['col1_width']
        col2_width = self.cleaned_data['col2_width']
        col3_width = self.cleaned_data['col3_width']
        if not (int(col1_width) + int(col2_width) + int(col3_width)) <= 12:
            raise forms.ValidationError(_("Summary of dimensions must be 12"))

    def clean(self):
        cleaned = super(PageDimensionForm, self).clean()
        self._check_dimensions()
        return cleaned

    class Meta:
        model = PageDimension
        exclude = tuple()
        widgets = {'page': forms.HiddenInput}




import django.dispatch
from dbtemplates.conf import settings
from dbtemplates.models import Template

try:
    from django.utils.timezone import now
except ImportError:
    from datetime import datetime
    now = datetime.now


def dbtemplate_save(sender, instance, created, **kwargs):
    """create widget/page content/base theme from given db template::
        /widget/icon/my_awesome.html
        /base/widget/my_new_widget_box.html
        /base/page/my_new_page_layout.html
    """

    if created:
        if 'widget' in instance.name:
            name = instance.name.split('/')[-1]
            kwargs = {
                'name': name.split('.')[0],
                'label': name.split('.')[0].capitalize(),
                'template': instance,
            }
            if 'base/widget' in instance.name:
                from leonardo.module.web.models import WidgetBaseTheme
                theme_cls = WidgetBaseTheme
            else:
                from leonardo.module.web.models import WidgetContentTheme
                theme_cls = WidgetContentTheme
                from leonardo.utils.widgets import find_widget_class
                w_cls_name = instance.name.split('/')[-2]
                w_cls = find_widget_class(w_cls_name)
                if w_cls is None:
                    raise Exception('widget class for %s not found' % w_cls_name)
                kwargs['widget_class'] = w_cls.__name__
            theme_cls(**kwargs).save()

        if 'base/page' in instance.name:
            from leonardo.module.web.models import PageTheme
            page_theme = PageTheme()
            page_theme.label = '{} layout'.format(
                instance.name.split("/")[-1].split('.')[0].title())
            page_theme.name = instance.name.split("/")[-1]
            page_theme.template = instance
            page_theme.save()

template_post_save = django.dispatch.Signal(providing_args=["instance", "created"])
template_post_save.connect(dbtemplate_save, dispatch_uid="sync_themes")


def save(self, *args, **kwargs):
    self.last_changed = now()
    # If content is empty look for a template with the given name and
    # populate the template instance with its content.
    if settings.DBTEMPLATES_AUTO_POPULATE_CONTENT and not self.content:
        self.populate()

    sync_themes = kwargs.pop('sync_themes', True)
    created = True
    if self.pk:
        created = False

    super(Template, self).save(*args, **kwargs)

    if sync_themes:
        from leonardo.module.web.signals import template_post_save
        template_post_save.send(
            sender=self.__class__, instance=self, created=created)




from .file import (DocumentField, FileField, FileMultipleField,
                   MultipleDocumentField, MultipleVectorField,
                   MultipleVideoField, VectorField, VideoField)
from .folder import FolderField, FolderMultipleField
from .image import ImageField, ImageMultipleField

__all__ = ['DocumentField', 'FileField', 'FileMultipleField',
           'MultipleDocumentField,' 'MultipleVectorField',
           'MultipleVideoField', 'VectorField', 'VideoField',
           'FolderField', 'FolderMultipleField',
           'ImageField', 'ImageMultipleField']



"""

code from https://github.com/TyMaszWeb/django-template-finder

"""
import fnmatch
import logging
import os
import re

from django.conf import settings
from django.utils.text import capfirst


try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module


try:
    from django.utils.six import string_types
except ImportError:
    string_types = (str,)
try:
    from django.template import Engine
except ImportError:
    class Engine(object):
        @staticmethod
        def get_default():
            return None


__all__ = ('find_all_templates', 'flatten_template_loaders', 'template_choices')


LOGGER = logging.getLogger('templatefinder')


def find_all_templates(pattern='*.html', ignore_private=True):
    """
    Finds all Django templates matching given glob in all TEMPLATE_LOADERS

    :param str pattern: `glob <http://docs.python.org/2/library/glob.html>`_
                        to match

    .. important:: At the moment egg loader is not supported.
    """
    templates = []
    template_loaders = flatten_template_loaders(settings.TEMPLATE_LOADERS)
    for loader_name in template_loaders:
        module, klass = loader_name.rsplit('.', 1)
        if loader_name in (
            'django.template.loaders.app_directories.Loader',
            'django.template.loaders.filesystem.Loader',
        ):
            loader_class = getattr(import_module(module), klass)
            if getattr(loader_class, '_accepts_engine_in_init', False):
                loader = loader_class(Engine.get_default())
            else:
                loader = loader_class()
            for dir in loader.get_template_sources(''):
                for root, dirnames, filenames in os.walk(dir):
                    for basename in filenames:
                        if ignore_private and basename.startswith("_"):
                            continue
                        filename = os.path.join(root, basename)
                        rel_filename = filename[len(dir)+1:]
                        if fnmatch.fnmatch(filename, pattern) or \
                           fnmatch.fnmatch(basename, pattern) or \
                           fnmatch.fnmatch(rel_filename, pattern):
                            templates.append(rel_filename)
        else:
            LOGGER.debug('%s is not supported' % loader_name)
    return sorted(set(templates))


def flatten_template_loaders(templates):
    """
    Given a collection of template loaders, unwrap them into one flat iterable.

    :param templates: template loaders to unwrap
    :return: template loaders as an iterable of strings.
    :rtype: generator expression
    """
    for loader in templates:
        if not isinstance(loader, string_types):
            for subloader in flatten_template_loaders(loader):
                yield subloader
        else:
            yield loader


def template_choices(templates, display_names=None, suffix=False):
    """
    Given an iterable of `templates`, calculate human-friendly display names
    for each of them, optionally using the `display_names` provided, or a
    global dictionary (`TEMPLATEFINDER_DISPLAY_NAMES`) stored in the Django
    project's settings.

    .. note:: As the resulting iterable is a lazy generator, if it needs to be
              consumed more than once, it should be turned into a `set`, `tuple`
              or `list`.

    :param list templates: an iterable of template paths, as returned by
                           `find_all_templates`
    :param display_names: If given, should be a dictionary where each key
                          represents a template path in `templates`, and each
                          value is the display text.
    :type display_names: dictionary or None
    :return: an iterable of two-tuples representing value (0) & display text (1)
    :rtype: generator expression
    """
    # allow for global template names, as well as usage-local ones.
    if display_names is None:
        display_names = getattr(settings, 'TEMPLATEFINDER_DISPLAY_NAMES', {})

    to_space_re = re.compile(r'[^a-zA-Z0-9\-]+')

    def fix_display_title(template_path):
        if template_path in display_names:
            return display_names[template_path]
        # take the last part from the template path; works even if there is no /
        lastpart = template_path.rpartition('/')[-1]
        # take everything to the left of the rightmost . (the file extension)
        if suffix:
            lastpart_with_suffix = lastpart
            return capfirst(lastpart_with_suffix)
        else:
            lastpart_minus_suffix = lastpart.rpartition('.')[0]
            # convert most non-alphanumeric characters into spaces, with the
            # exception of hyphens.
            lastpart_spaces = to_space_re.sub(' ', lastpart_minus_suffix)
        return capfirst(lastpart_spaces)

    return ((template, fix_display_title(template)) for template in templates)



from django.core import urlresolvers
from django.utils.translation import ugettext_lazy as _
from . import mixins
from ..models import File
from ..models import Folder
from .. import settings as filer_settings


class DummyFolder(mixins.IconsMixin):
    file_type = 'DummyFolder'
    name = "Dummy Folder"
    is_root = True
    is_smart_folder = True
    can_have_subfolders = False
    parent = None
    _icon = "plainfolder"

    @property
    def virtual_folders(self):
        return []

    @property
    def media_folder_children(self):
        return Folder.objects.none()

    @property
    def files(self):
        return File.objects.none()
    parent_url = None

    @property
    def image_files(self):
        return self.files

    @property
    def logical_path(self):
        """
        Gets logical path of the folder in the tree structure.
        Used to generate breadcrumbs
        """
        return []


class UnfiledImages(DummyFolder):
    name = _("unfiled files")
    is_root = True
    _icon = "unfiled_folder"

    def _files(self):
        return File.objects.filter(folder__isnull=True)
    files = property(_files)

    def get_admin_directory_listing_url_path(self):
        return urlresolvers.reverse(
                            'admin:filer-directory_listing-unfiled_images')


class ImagesWithMissingData(DummyFolder):
    name = _("files with missing metadata")
    is_root = True
    _icon = "incomplete_metadata_folder"

    @property
    def files(self):
        return File.objects.filter(has_all_mandatory_data=False)

    def get_admin_directory_listing_url_path(self):
        return urlresolvers.reverse(
                    'admin:filer-directory_listing-images_with_missing_data')


class FolderRoot(DummyFolder):
    name = _('root')
    is_root = True
    is_smart_folder = False
    can_have_subfolders = True

    @property
    def virtual_folders(self):
        return [UnfiledImages()]

    @property
    def children(self):
        if filer_settings.FILER_ENABLE_PERMISSIONS:
            return Folder.objects.all()
        return Folder.objects.filter(parent__isnull=True)
    parent_url = None

    @property
    def media_folder_children(self):
        return self.children

    def contains_folder(self, folder_name):
        try:
            self.children.get(name=folder_name)
            return True
        except Folder.DoesNotExist:
            return False

    def get_admin_directory_listing_url_path(self):
        return urlresolvers.reverse('admin:filer-directory_listing-root')



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.media.fields import FileField
from leonardo.module.web.models import Widget
from leonardo.module.web.widgets.forms import WidgetUpdateForm


class FileForm(WidgetUpdateForm):

    file = FileField()


class DownloadItemWidget(Widget):

    feincms_item_editor_form = FileForm

    icon = "fa fa-download"

    file = models.ForeignKey("media.File", verbose_name=_("file"),
                             related_name="%(app_label)s_%(class)s_files")

    class Meta:
        abstract = True
        verbose_name = _("download item")
        verbose_name_plural = _('download items')




from __future__ import absolute_import

from django.conf.urls import patterns, url

from .views import *


urlpatterns = patterns('',
                       url(r'^(?P<page_id>[\w\.\-]+)/(?P<region>[\w\.\-]+)/(?P<cls_name>[\w\.\-]+)/select-widget/$',
                           WidgetPreCreateView.as_view(), name='widget_create'),
                       url(r'^(?P<page_id>[\w\.\-]+)/(?P<region>[\w\.\-]+)/(?P<cls_name>[\w\.\-]+)/(?P<ordering>[\w\.\-]+)/(?P<parent>[\w\.\-]+)/create/$',
                           WidgetCreateView.as_view(), name='widget_create_full'),
                       url(r'^(?P<cls_name>[\w\.\-]+)/(?P<id>[\w\.\-]+)/update/$',
                           WidgetUpdateView.as_view(), name='widget_update'),
                       url(r'^(?P<cls_name>[\w\.\-]+)/(?P<id>[\w\.\-]+)/delete/$',
                           WidgetDeleteView.as_view(), name='widget_delete'),
                       url(r'^(?P<cls_name>[\w\.\-]+)/(?P<id>[\w\.\-]+)/info/$',
                           WidgetInfoView.as_view(), name='widget_info'),
                       url(r'^(?P<cls_name>[\w\.\-]+)/(?P<id>[\w\.\-]+)/(?P<ordering>[\w\.\-]+)/ordering/$',
                           WidgetReorderView.as_view(), name='widget_reorder'),
                       url(r'^widget-sort/$',
                           WidgetSortView.as_view(), name='widget_sort'),
                       )




from __future__ import absolute_import

from django.conf import settings
from horizon import conf

from ..models import Page


class WebMiddleware(object):

    """add extra context to request

    added some extra to request and page

    .. code-block:: python

        request.leonardo_options

        request.LEONARDO_CONFIG

        request.leonardo_page

    supports this syntax

    .. code-block:: python

        request.LEONARDO_CONFIG.DISQUS_COMMENTS

    note: for support old ``webcms`` stuff adds some
    extra stuff which would be old after migration

    """

    def process_response(self, request, response):

        if hasattr(request, 'user') and not request.user.is_authenticated():
            response.delete_cookie('frontend_editing')
            request.frontend_editing = False
        return response

    def process_request(self, request):
        try:
            leonardo_options = {
                'meta_description': settings.META_DESCRIPTION,
                'meta_keywords': settings.META_KEYWORDS,
                'meta_title': settings.META_TITLE,
            }
        except:
            leonardo_options = {
                'meta_description': '',
                'meta_keywords': '',
                'meta_title': '',
            }

        leonardo_options['site'] = {
            'name': settings.SITE_NAME,
            'id': settings.SITE_ID,
            'domain': getattr(
                settings, 'SITE_DOMAIN', settings.SITE_NAME + '.cz'),
        }

        try:
            page = Page.objects.best_match_for_path(
                request.path)
        except Exception:
            page = None
            leonardo_options['template'] = 'base.html'
            leonardo_options['theme'] = 'light'
            leonardo_options['assets'] = []
            leonardo_options['widgets'] = []

        request.leonardo_options = leonardo_options
        request.leonardo_page = page

        request.frontend_editing = request.COOKIES.get(
            'frontend_editing', False)

        request.LEONARDO_CONFIG = conf.HORIZON_CONFIG

        # old
        request.webcms_page = page
        request.webcms_options = leonardo_options




from .breadcrumbs.models import BreadcrumbsWidget
from .contentnavigation.models import ContentNavigationWidget
from .contextnavigation.models import ContextNavigationWidget
from .linearnavigation.models import LinearNavigationWidget
from .sitemap.models import SiteMapWidget
from .sitesearch.models import SiteSearchWidget
from .treenavigation.models import TreeNavigationWidget
from .languageselector.models import LanguageSelectorWidget




from __future__ import unicode_literals

from .multiple import MultiSelectField

from django_select2.forms import *


class InitMixin(object):

    '''Support for declaring Model Select Field'''

    def __init__(self, *args, **kwargs):
        super(FileField, self).__init__(
            self.queryset, self.empty_label, *args, **kwargs)


class Select2Field(ModelSelect2Widget, InitMixin):

    pass


class Select2MultipleField(ModelSelect2MultipleWidget, InitMixin):

    pass



from __future__ import absolute_import

from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.contrib.auth.tokens import default_token_generator
from django.contrib.sites.models import Site
from django.core import exceptions
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import pgettext, ugettext
from horizon import forms, messages
from leonardo.forms import SelfHandlingForm, Layout, InlineCheckboxes
from horizon.utils import validators


class LoginForm(SelfHandlingForm):

    username = forms.CharField(label=_("Username"),
                               max_length=255,
                               widget=forms.TextInput(
                                   attrs={'placeholder':
                                          _('Username'),
                                          'autofocus': 'autofocus'}))

    password = forms.CharField(label=_("Password"),
                               widget=forms.PasswordInput(render_value=False))
    remember = forms.BooleanField(label=_("Remember Me"),
                                  required=False)

    def __init__(self, *args, **kwargs):
        super(LoginForm, self).__init__(*args, **kwargs)
        self.helper.layout = Layout(
            'username', 'password', 'remember',
        )

    def handle(self, request, data):

        user = authenticate(**data)
        if user is not None:
            if user.is_active:
                login(request, user)

                messages.success(request, "Login success.")
                return True
        messages.error(request, "Login failed.")
        return False


class SignupForm(SelfHandlingForm):

    username = forms.CharField(label=_("Username"),
                               max_length=255,
                               widget=forms.TextInput(
                                   attrs={'placeholder':
                                          _('Username'),
                                          'autofocus': 'autofocus'}))
    email = forms.EmailField(widget=forms.TextInput(
        attrs={'type': 'email',
               'placeholder': _('E-mail address')}))

    password = forms.RegexField(
        label=_("Password"),
        widget=forms.PasswordInput(render_value=False),
        regex=validators.password_validator(),
        error_messages={'invalid': validators.password_validator_msg()})
    confirm_password = forms.CharField(
        label=_("Confirm Password"),
        widget=forms.PasswordInput(render_value=False))
    no_autocomplete = True

    def clean(self):
        '''Check to make sure password fields match.'''
        data = super(forms.Form, self).clean()

        # basic check for now
        if 'username' in data:
            if User.objects.filter(
                    username=data['username'],
                    email=data['email']).exists():
                raise validators.ValidationError(
                    _('Username or email exists in database.'))

        if 'password' in data:
            if data['password'] != data.get('confirm_password', None):
                raise validators.ValidationError(_('Passwords do not match.'))
            else:
                data.pop('confirm_password')
        return data

    def handle(self, request, data):

        try:
            user = User.objects.create_user(**data)
            messages.success(
                request,
                _("User account {} was successfuly created.".format(user)))

        except Exception as e:
            raise e
        else:
            data.pop('email')
            return LoginForm().handle(request, data)

        messages.error(request, _("Create Account failed."))
        return False


"""
class ChangePasswordForm(UserForm):

    oldpassword = PasswordField(label=_("Current Password"))
    password1 = SetPasswordField(label=_("New Password"))
    password2 = PasswordField(label=_("New Password (again)"))

    def clean_oldpassword(self):
        if not self.user.check_password(self.cleaned_data.get("oldpassword")):
            raise forms.ValidationError(_("Please type your current"
                                          " password."))
        return self.cleaned_data["oldpassword"]

    def clean_password2(self):
        if ("password1" in self.cleaned_data
                and "password2" in self.cleaned_data):
            if (self.cleaned_data["password1"]
                    != self.cleaned_data["password2"]):
                raise forms.ValidationError(_("You must type the same password"
                                              " each time."))
        return self.cleaned_data["password2"]

    def save(self):
        get_adapter().set_password(self.user, self.cleaned_data["password1"])


class SetPasswordForm(UserForm):

    password1 = SetPasswordField(label=_("Password"))
    password2 = PasswordField(label=_("Password (again)"))

    def clean_password2(self):
        if ("password1" in self.cleaned_data
                and "password2" in self.cleaned_data):
            if (self.cleaned_data["password1"]
                    != self.cleaned_data["password2"]):
                raise forms.ValidationError(_("You must type the same password"
                                              " each time."))
        return self.cleaned_data["password2"]

    def save(self):
        get_adapter().set_password(self.user, self.cleaned_data["password1"])


class ResetPasswordForm(forms.Form):

    email = forms.EmailField(
        label=_("E-mail"),
        required=True,
        widget=forms.TextInput(attrs={"type": "email", "size": "30"}))

    def clean_email(self):
        email = self.cleaned_data["email"]
        email = get_adapter().clean_email(email)
        self.users = get_user_model().objects \
            .filter(Q(email__iexact=email)
                    | Q(emailaddress__email__iexact=email)).distinct()
        if not self.users.exists():
            raise forms.ValidationError(_("The e-mail address is not assigned"
                                          " to any user account"))
        return self.cleaned_data["email"]

    def save(self, request, **kwargs):

        email = self.cleaned_data["email"]
        token_generator = kwargs.get("token_generator",
                                     default_token_generator)

        for user in self.users:

            temp_key = token_generator.make_token(user)

            # save it to the password reset model
            # password_reset = PasswordReset(user=user, temp_key=temp_key)
            # password_reset.save()

            current_site = Site.objects.get_current()

            # send the password reset email
            path = reverse("account_reset_password_from_key",
                           kwargs=dict(uidb36=user_pk_to_url_str(user),
                                       key=temp_key))
            url = build_absolute_uri(request, path,
                                     protocol=app_settings.DEFAULT_HTTP_PROTOCOL)
            context = {"site": current_site,
                       "user": user,
                       "password_reset_url": url}
            if app_settings.AUTHENTICATION_METHOD \
                    != AuthenticationMethod.EMAIL:
                context['username'] = user_username(user)
            get_adapter().send_mail('account/email/password_reset_key',
                                    email,
                                    context)
        return self.cleaned_data["email"]


class ResetPasswordKeyForm(forms.Form):

    password1 = SetPasswordField(label=_("New Password"))
    password2 = PasswordField(label=_("New Password (again)"))

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user", None)
        self.temp_key = kwargs.pop("temp_key", None)
        super(ResetPasswordKeyForm, self).__init__(*args, **kwargs)

    # FIXME: Inspecting other fields -> should be put in def clean(self) ?
    def clean_password2(self):
        if ("password1" in self.cleaned_data
                and "password2" in self.cleaned_data):
            if (self.cleaned_data["password1"]
                    != self.cleaned_data["password2"]):
                raise forms.ValidationError(_("You must type the same"
                                              " password each time."))
        return self.cleaned_data["password2"]

    def save(self):
        get_adapter().set_password(self.user, self.cleaned_data["password1"])
"""



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from leonardo.module.nav.models import NavigationWidget


TRAVERSE_CHOICES = (
    (0, _("none")),
    (1, _("parents")),
    (2, _("siblings")),
    (3, _("cousins")),
)

LINK_CHOICES = (
    ('text', _("previous/next")),
    ('page', _("page title")),
)


class LinearNavigationWidget(NavigationWidget):
    traverse = models.IntegerField(
        verbose_name=_("Node traversal"), choices=TRAVERSE_CHOICES, default=0)
    link_style = models.CharField(
        max_length=255, verbose_name=_("Link style"), choices=LINK_CHOICES, default='text')

    class Meta:
        abstract = True
        verbose_name = _("Linear pager")
        verbose_name_plural = _('Linear pagers')

    def render_content(self, options):
        request = options['request']
        page = request.webcms_page

        if self.traverse == 0:
            prev = page.get_previous_sibling()
            next = page.get_next_sibling()
        elif self.traverse == 1:
            prev = page.get_previous_sibling()
            if prev is None:
                prev = page
            next = page.get_next_sibling()
        else:
            prev = page.get_previous_sibling()
            next = page.get_next_sibling()
        return render_to_string(self.get_template_name(), {
            'widget': self,
            'request': request,
            'prev': prev,
            'next': next,
        })




from __future__ import absolute_import

from django.conf.urls import include, patterns, url

from .views import *

urlpatterns = patterns('',
                       url(r'^(?P<page_id>[\w\.\-]+)/theme$',
                           PageMassUpdateView.as_view(), name='page_mass_update'),
                       )




from django.conf import settings

from django.core.files.storage import get_storage_class
from filer.utils.loader import load_object
from filer.utils.recursive_dictionary import RecursiveDictionaryWithExcludes
import os



# File download permissions are an experimental
# feature. The api may change at any time.
MEDIA_ENABLE_PERMISSIONS = getattr(settings, 'MEDIA_ENABLE_PERMISSIONS', True)
MEDIA_IS_PUBLIC_DEFAULT = getattr(settings, 'MEDIA_IS_PUBLIC_DEFAULT', True)
MEDIA_PUBLIC_UPLOAD_TO = getattr(settings, 'MEDIA_PUBLIC_UPLOAD_TO', 'public')
MEDIA_PRIVATE_UPLOAD_TO = getattr(settings, 'MEDIA_PRIVATE_UPLOAD_TO', 'private')
MEDIA_PAGINATE_BY = getattr(settings, 'MEDIA_PAGINATE_BY', 25)
MEDIA_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS = getattr(settings, 'MEDIA_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS', False)

MEDIA_IMAGE_MODEL = getattr(settings, 'MEDIA_IMAGE_MODEL', False)
# This is an ordered iterable that describes a list of
# classes that I should check for when adding files
MEDIA_FILE_MODELS = getattr(settings, 'MEDIA_FILE_MODELS',
    (
        MEDIA_IMAGE_MODEL if MEDIA_IMAGE_MODEL else 'leonardo.module.media.models.Image',
        'leonardo.module.media.models.Document',
        'leonardo.module.media.models.Video',
        'leonardo.module.media.models.Vector',
        'leonardo.module.media.models.File',
    )
)

DEFAULT_FILE_STORAGE = getattr(settings, 'DEFAULT_FILE_STORAGE', 'django.core.files.storage.FileSystemStorage')


DEFAULT_MEDIA_STORAGES = {
    'public': {
        'main': {
            'ENGINE': DEFAULT_FILE_STORAGE,
            'OPTIONS': {
                'location': os.path.abspath(os.path.join(settings.MEDIA_ROOT)),
                'base_url': '/media/',
            },
            'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
            'UPLOAD_TO_PREFIX': MEDIA_PUBLIC_UPLOAD_TO,
        },
        'thumbnails': {
            'ENGINE': DEFAULT_FILE_STORAGE,
            'OPTIONS': {},
            'THUMBNAIL_OPTIONS': {
                'base_dir': 'public_thumbnails',
            },
        },
    },
    'private': {
        'main': {
            'ENGINE': 'filer.storage.PrivateFileSystemStorage',
            'OPTIONS': {
                'location': os.path.abspath(os.path.join(settings.MEDIA_ROOT)),
                'base_url': '/smedia/',
            },
            'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
            'UPLOAD_TO_PREFIX': MEDIA_PRIVATE_UPLOAD_TO,
        },
        'thumbnails': {
            'ENGINE': 'filer.storage.PrivateFileSystemStorage',
            'OPTIONS': {
                'location': os.path.abspath(os.path.join(settings.MEDIA_ROOT, MEDIA_PRIVATE_UPLOAD_TO)),
                'base_url': '/smedia/private_thumbnails/',
            },
            'THUMBNAIL_OPTIONS': {},
        },
    },
}

# OLD FILER STUFF

DEFAULT_FILER_STORAGES = DEFAULT_MEDIA_STORAGES

FILER_IMAGE_MODEL = MEDIA_IMAGE_MODEL
FILER_ENABLE_PERMISSIONS = MEDIA_ENABLE_PERMISSIONS
FILER_DEBUG = getattr(settings, 'FILER_DEBUG', False) # When True makes
FILER_SUBJECT_LOCATION_IMAGE_DEBUG = getattr(settings, 'FILER_SUBJECT_LOCATION_IMAGE_DEBUG', False)
FILER_WHITESPACE_COLOR = getattr(settings, 'FILER_WHITESPACE_COLOR', '#FFFFFF')

FILER_ENABLE_LOGGING = getattr(settings, 'FILER_ENABLE_LOGGING', False)
if FILER_ENABLE_LOGGING:
    FILER_ENABLE_LOGGING = (FILER_ENABLE_LOGGING and (getattr(settings,'LOGGING') and
                                                      ('' in settings.LOGGING['loggers'] or
                                                       'filer' in settings.LOGGING['loggers'])))

FILER_IS_PUBLIC_DEFAULT = MEDIA_IS_PUBLIC_DEFAULT

FILER_PAGINATE_BY = MEDIA_PAGINATE_BY

FILER_STATICMEDIA_PREFIX = getattr(settings, 'FILER_STATICMEDIA_PREFIX', None)
if not FILER_STATICMEDIA_PREFIX:
    FILER_STATICMEDIA_PREFIX = (getattr(settings, 'STATIC_URL', None) or settings.MEDIA_URL) + 'filer/'

MEDIA_ADMIN_ICON_SIZES = getattr(settings,"MEDIA_ADMIN_ICON_SIZES",(
    '16', '32', '48', '64',
    ))
FILER_ADMIN_ICON_SIZES = MEDIA_ADMIN_ICON_SIZES

THUMBNAIL_HIGH_RESOLUTION = True

THUMBNAIL_PROCESSORS = (
    'easy_thumbnails.processors.colorspace',
    'easy_thumbnails.processors.autocrop',
    #'easy_thumbnails.processors.scale_and_crop',
    'filer.thumbnail_processors.scale_and_crop_with_subject_location',
    'easy_thumbnails.processors.filters',
)

FILER_FILE_MODELS = MEDIA_FILE_MODELS

MINIMAL_FILER_STORAGES = {
    'public': {
        'main': {
            'ENGINE': None,
            'OPTIONS': {},
            },
        'thumbnails': {
            'ENGINE': None,
            'OPTIONS': {},
            }
    },
    'private': {
        'main': {
            'ENGINE': None,
            'OPTIONS': {},
            },
        'thumbnails': {
            'ENGINE': None,
            'OPTIONS': {},
            },
        },
    }

MINIMAL_FILER_SERVERS = {
    'private': {
        'main': {
            'ENGINE': None,
            'OPTIONS': {},
        },
        'thumbnails': {
            'ENGINE': None,
            'OPTIONS': {},
        },
    },
}

DEFAULT_FILER_SERVERS = {
    'private': {
        'main': {
            'ENGINE': 'filer.server.backends.default.DefaultServer',
            'OPTIONS': {},
        },
        'thumbnails': {
            'ENGINE': 'filer.server.backends.default.DefaultServer',
            'OPTIONS': {},
        },
    },
}

FILER_STORAGES = RecursiveDictionaryWithExcludes(MINIMAL_FILER_STORAGES, rec_excluded_keys=('OPTIONS', 'THUMBNAIL_OPTIONS'))

user_filer_storages = getattr(settings, 'FILER_STORAGES', {})

FILER_STORAGES.rec_update(user_filer_storages)

def update_storage_settings(user_settings, defaults, s, t):
    if not user_settings[s][t]['ENGINE']:
        user_settings[s][t]['ENGINE'] = defaults[s][t]['ENGINE']
        user_settings[s][t]['OPTIONS'] = defaults[s][t]['OPTIONS']
    if t == 'main':
        if not 'UPLOAD_TO' in user_settings[s][t]:
            user_settings[s][t]['UPLOAD_TO'] = defaults[s][t]['UPLOAD_TO']
        if not 'UPLOAD_TO_PREFIX' in user_settings[s][t]:
            user_settings[s][t]['UPLOAD_TO_PREFIX'] = defaults[s][t]['UPLOAD_TO_PREFIX']
    if t == 'thumbnails':
        if not 'THUMBNAIL_OPTIONS' in user_settings[s][t]:
            user_settings[s][t]['THUMBNAIL_OPTIONS'] = defaults[s][t]['THUMBNAIL_OPTIONS']
    return user_settings

update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'public', 'main')
update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'public', 'thumbnails')
update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'private', 'main')
update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'private', 'thumbnails')

FILER_SERVERS = RecursiveDictionaryWithExcludes(MINIMAL_FILER_SERVERS, rec_excluded_keys=('OPTIONS',))
FILER_SERVERS.rec_update(getattr(settings, 'FILER_SERVERS', {}))

def update_server_settings(settings, defaults, s, t):
    if not settings[s][t]['ENGINE']:
        settings[s][t]['ENGINE'] = defaults[s][t]['ENGINE']
        settings[s][t]['OPTIONS'] = defaults[s][t]['OPTIONS']
    return settings

update_server_settings(FILER_SERVERS, DEFAULT_FILER_SERVERS, 'private', 'main')
update_server_settings(FILER_SERVERS, DEFAULT_FILER_SERVERS, 'private', 'thumbnails')



# Public media (media accessible without any permission checks)
FILER_PUBLICMEDIA_STORAGE = get_storage_class(FILER_STORAGES['public']['main']['ENGINE'])(**FILER_STORAGES['public']['main']['OPTIONS'])
FILER_PUBLICMEDIA_UPLOAD_TO = load_object(FILER_STORAGES['public']['main']['UPLOAD_TO'])
if 'UPLOAD_TO_PREFIX' in FILER_STORAGES['public']['main']:
    FILER_PUBLICMEDIA_UPLOAD_TO = load_object('filer.utils.generate_filename.prefixed_factory')(FILER_PUBLICMEDIA_UPLOAD_TO, FILER_STORAGES['public']['main']['UPLOAD_TO_PREFIX'])
FILER_PUBLICMEDIA_THUMBNAIL_STORAGE = get_storage_class(FILER_STORAGES['public']['thumbnails']['ENGINE'])(**FILER_STORAGES['public']['thumbnails']['OPTIONS'])
FILER_PUBLICMEDIA_THUMBNAIL_OPTIONS = FILER_STORAGES['public']['thumbnails']['THUMBNAIL_OPTIONS']


# Private media (media accessible through permissions checks)
FILER_PRIVATEMEDIA_STORAGE = get_storage_class(FILER_STORAGES['private']['main']['ENGINE'])(**FILER_STORAGES['private']['main']['OPTIONS'])
FILER_PRIVATEMEDIA_UPLOAD_TO = load_object(FILER_STORAGES['private']['main']['UPLOAD_TO'])
if 'UPLOAD_TO_PREFIX' in FILER_STORAGES['private']['main']:
    FILER_PRIVATEMEDIA_UPLOAD_TO = load_object('filer.utils.generate_filename.prefixed_factory')(FILER_PRIVATEMEDIA_UPLOAD_TO, FILER_STORAGES['private']['main']['UPLOAD_TO_PREFIX'])
FILER_PRIVATEMEDIA_THUMBNAIL_STORAGE = get_storage_class(FILER_STORAGES['private']['thumbnails']['ENGINE'])(**FILER_STORAGES['private']['thumbnails']['OPTIONS'])
FILER_PRIVATEMEDIA_THUMBNAIL_OPTIONS = FILER_STORAGES['private']['thumbnails']['THUMBNAIL_OPTIONS']
FILER_PRIVATEMEDIA_SERVER = load_object(FILER_SERVERS['private']['main']['ENGINE'])(**FILER_SERVERS['private']['main']['OPTIONS'])
FILER_PRIVATEMEDIA_THUMBNAIL_SERVER = load_object(FILER_SERVERS['private']['thumbnails']['ENGINE'])(**FILER_SERVERS['private']['thumbnails']['OPTIONS'])

FILER_DUMP_PAYLOAD = getattr(settings, 'FILER_DUMP_PAYLOAD', False)  # Whether the filer shall dump the files payload



from django.conf import settings
from django.template import Library

register = Library()


def filer_actions(context):
    """
    Track the number of times the action field has been rendered on the page,
    so we know which value to use.
    """
    context['action_index'] = context.get('action_index', -1) + 1
    return context
filer_actions = register.inclusion_tag(
    "admin/media/actions.html", takes_context=True)(filer_actions)


ADMIN_ICON_BASE = "%sadmin/img/" % settings.STATIC_URL
ADMIN_CSS_BASE = "%sadmin/css/" % settings.STATIC_URL
ADMIN_JS_BASE = "%sadmin/js/" % settings.STATIC_URL


@register.simple_tag
def admin_icon_base():
    return ADMIN_ICON_BASE


@register.simple_tag
def admin_css_base():
    return ADMIN_CSS_BASE


@register.simple_tag
def admin_js_base():
    return ADMIN_JS_BASE
